@@ -33,6 +33,21 @@
 extern mglTeXsymb mgl_tex_symb[];
 int test(mglGraphAB *gr)
 {
+mglTestMode=true;
+	mreal a[100];   // let a_i = sin(4*pi*x), x=0...1
+	for(int i=0;i<100;i++)a[i]=sin(4*M_PI*i/99);
+	mglParse *parser = new mglParse;
+	mglData &d =(parser->AddVar("dat"))->d;
+	d.Set(a,100); // set data to variable
+	parser->Execute(gr, "plot dat; xrange 0 1\nbox\naxis");
+	// you may break script at any line do something
+	// and continue after that
+	parser->Execute(gr, "xlabel 'x'\nylabel 'y'\nbox");
+	// also you may use cycles or conditions in script
+	parser->Execute(gr, "for $0 -1 1 0.1\nline 0 0 -1 $0 'r'\nnext");
+	delete parser;
+	return 0;
+
 	mglParse par;
 	par.AllowSetSize = true;
 	FILE *fp=fopen("test.mgl","rt");
@@ -189,6 +204,7 @@ void smgl_fonts(mglGraph *gr)	// ticks features
 	gr->LoadFont("pagella");	gr->Puts(mglPoint(0,h-7*d), "pagella font");
 	gr->LoadFont("schola");		gr->Puts(mglPoint(0,h-8*d), "schola font");
 	gr->LoadFont("termes");		gr->Puts(mglPoint(0,h-9*d), "termes font");
+	gr->LoadFont("STIX");
 }
 //-----------------------------------------------------------------------------
 void smgl_stick(mglGraph *gr)	// column plot
@@ -1506,7 +1522,7 @@ void smgl_ternary(mglGraph *gr)	// flag #
 	rx.Modify("rnd"); ry.Modify("(1-v)*rnd",rx);
 
 	gr->Text(mglPoint(-0.8,1.3), "Ternary plot (x+y+t=1)");
-	gr->Ternary(true);
+	gr->Ternary(1);
 	gr->Plot(x,y,"r2");
 	gr->Plot(rx,ry,"q^ ");
 	gr->Cont(a,"BbcyrR");
@@ -1515,7 +1531,26 @@ void smgl_ternary(mglGraph *gr)	// flag #
 	gr->Label('x',"x comp.");
 	gr->Label('y',"y comp.");
 	gr->Label('t',"t comp.");
-	gr->Ternary(false);
+	gr->Ternary(0);
+}
+//-----------------------------------------------------------------------------
+void smgl_quaternary(mglGraph *gr)	// flag #
+{
+	mglData x(50),y(50),z(50),rx(10),ry(10), a(20,30);
+	a.Modify("x*y*(1-x-y)^2*30");
+	x.Modify("0.25*(1-x)*(1+cos(2*pi*x))");
+	y.Modify("0.25*(1-x)*(1+sin(2*pi*x))");
+	z.Modify("x");
+
+	gr->Text(mglPoint(0,1.3), "Quaternary plot (x+y+z+t=1)");
+	gr->Rotate(60,70);		gr->Light(true);
+	gr->Ternary(2);			gr->SetCut(true);
+	gr->Plot(x,y,z,"r2");	gr->Surf(a,"BbcyrR#");
+	gr->Axis(); gr->Grid();	gr->Box();
+	gr->SetRotatedText(false);
+	gr->Label('t',"A",1);	gr->Label('x',"B",1);
+	gr->Label('y',"C",1);	gr->Label('z',"D",1);
+	gr->Ternary(0);
 }
 //-----------------------------------------------------------------------------
 void smgl_drops(mglGraph *gr)	// flag #
@@ -1885,6 +1920,7 @@ mglSample samp[] = {
 	{"pipe3", smgl_pipe3},
 	{"plot", smgl_plot},
 	{"qo2d", smgl_qo2d},
+	{"quaternary", smgl_quaternary},
 	{"radar", smgl_radar},
 	{"region", smgl_region},
 	{"region_2", smgl_region_2},
