@@ -27,8 +27,32 @@ int sample_2(mglGraph *gr, void *);
 int sample_3(mglGraph *gr, void *);
 int sample_d(mglGraph *gr, void *);
 //-----------------------------------------------------------------------------
+#include <pthread.h>
+#include <unistd.h>
+mglPoint pnt;  // some global variable for changable data
+void *mgl_qt_tmp(void *)	{	mglQtRun();	return 0;	}
+//-----------------------------------------------------------------------------
 int main(int argc,char **argv)
 {
+#ifdef PTHREAD_SAMPLE
+	mglGraphQT gr;
+	gr.Window(argc,argv,NULL,"test",0,0);  // create window
+	gr.ClfOnUpdate = false;
+	static pthread_t tmp;
+	pthread_create(&tmp, 0, mgl_qt_tmp, 0);
+	pthread_detach(tmp);    // run window handling in the separate thread
+	for(int i=0;i<10;i++)   // do calculation
+	{
+		sleep(2);             // which can be very long
+		pnt = mglPoint(2*mgl_rnd()-1,2*mgl_rnd()-1);
+		gr.Clf();             // make new drawing
+		gr.Line(mglPoint(),pnt,"Ar2");
+		char str[10] = "i=0";	str[3] = '0'+i;
+		gr.Text(mglPoint(),"");
+		gr.Update();       // update window
+	}
+	return 0;   // finish calculations and close the window
+#else
 	mglGraphQT gr;
 	char key = 0;
 	if(argc>1 && argv[1][0]!='-')	key = argv[1][0];
@@ -43,5 +67,6 @@ int main(int argc,char **argv)
 	default:	gr.Window(argc,argv,sample,"Example of molecules");	break;
 	}
 	return mglQtRun();
+#endif
 }
 //-----------------------------------------------------------------------------
