@@ -71,6 +71,22 @@ HMGL mgl_create_graph_fltk_dr(HMDR dr, const char *title);
 HMGL mgl_create_graph_qt_dr(HMDR dr, const char *title);
 void mgl_fltk_run();
 void mgl_qt_run();
+
+void mgl_wnd_set_delay(HMGL gr, mreal dt);
+void mgl_wnd_set_auto_clf(HMGL gr, int val);
+void mgl_wnd_set_show_mouse_pos(HMGL gr, int val);
+void mgl_wnd_set_clf_update(HMGL gr, int val);
+void mgl_wnd_toggle_alpha(HMGL gr);
+void mgl_wnd_toggle_light(HMGL gr);
+void mgl_wnd_toggle_zoom(HMGL gr);
+void mgl_wnd_toggle_rotate(HMGL gr);
+void mgl_wnd_toggle_no(HMGL gr);
+void mgl_wnd_update(HMGL gr);
+void mgl_wnd_reload(HMGL gr, int o);
+void mgl_wnd_adjust(HMGL gr);
+void mgl_wnd_next_frame(HMGL gr);
+void mgl_wnd_prev_frame(HMGL gr);
+void mgl_wnd_animation(HMGL gr);
 #endif
 void mgl_set_show_mouse_pos(HMGL gr, int enable);
 void mgl_get_last_mouse_pos(HMGL gr, mreal *x, mreal *y, mreal *z);
@@ -88,6 +104,7 @@ void mgl_delete_data(HMDT dat);
 /*****************************************************************************/
 HMPR mgl_create_parser();
 void mgl_delete_parser(HMPR p);
+void mgl_scan_func(HMPR p, const wchar_t *line);
 void mgl_add_param(HMPR p, int id, const char *str);
 void mgl_add_paramw(HMPR p, int id, const wchar_t *str);
 /*===!!! NOTE !!! You must not delete obtained data arrays !!!===============*/
@@ -129,6 +146,7 @@ void mgl_set_scheme(HMGL gr, const char *sch);
 void mgl_load_font(HMGL gr, const char *name, const char *path);
 void mgl_copy_font(HMGL gr, HMGL gr_from);
 void mgl_restore_font(HMGL gr);
+int mgl_get_warn(HMGL gr);
 /*****************************************************************************/
 /*		Export to file or to memory											 */
 /*****************************************************************************/
@@ -167,15 +185,19 @@ void mgl_set_ambbr(HMGL gr, mreal i);
 /*****************************************************************************/
 /*		Scale and rotate													 */
 /*****************************************************************************/
+void mgl_mat_pop(HMGL gr);
+void mgl_mat_push(HMGL gr);
 void mgl_identity(HMGL graph, int rel);
 void mgl_clf(HMGL graph);
 void mgl_flush(HMGL gr);
 void mgl_clf_rgb(HMGL graph, mreal r, mreal g, mreal b);
 void mgl_subplot(HMGL graph, int nx,int ny,int m);
 void mgl_subplot_d(HMGL graph, int nx,int ny,int m, mreal dx, mreal dy);
+void mgl_subplot_s(HMGL graph, int nx,int ny,int m,const char *style);
 void mgl_inplot(HMGL graph, mreal x1,mreal x2,mreal y1,mreal y2);
 void mgl_relplot(HMGL graph, mreal x1,mreal x2,mreal y1,mreal y2);
 void mgl_columnplot(HMGL graph, int num, int ind);
+void mgl_columnplot_d(HMGL graph, int num, int ind, mreal d);
 void mgl_stickplot(HMGL graph, int num, int ind, mreal tet, mreal phi);
 void mgl_aspect(HMGL graph, mreal Ax,mreal Ay,mreal Az);
 void mgl_rotate(HMGL graph, mreal TetX,mreal TetZ,mreal TetY);
@@ -194,6 +216,8 @@ void mgl_set_caxis(HMGL graph, mreal C1,mreal C2);
 void mgl_set_axis(HMGL graph, mreal x1, mreal y1, mreal z1, mreal x2, mreal y2, mreal z2, mreal x0, mreal y0, mreal z0);
 void mgl_set_axis_3d(HMGL graph, mreal x1, mreal y1, mreal z1, mreal x2, mreal y2, mreal z2);
 void mgl_set_axis_2d(HMGL graph, mreal x1, mreal y1, mreal x2, mreal y2);
+inline void mgl_set_ranges(HMGL graph, mreal x1, mreal x2, mreal y1, mreal y2, mreal z1, mreal z2)
+{	mgl_set_axis_3d(graph, x1,y1,z1,x2,y2,z2);	};
 void mgl_set_origin(HMGL graph, mreal x0, mreal y0, mreal z0);
 void mgl_set_tick_origin(HMGL graph, mreal x0, mreal y0, mreal z0);
 void mgl_set_crange(HMGL graph, const HMDT a, int add);
@@ -257,6 +281,7 @@ void mgl_clear_legend(HMGL graph);
 void mgl_legend_xy(HMGL graph, mreal x, mreal y, const char *font, mreal size, mreal llen);
 void mgl_legend(HMGL graph, int where, const char *font, mreal size, mreal llen);
 void mgl_set_legend_box(HMGL gr, int enable);
+void mgl_set_legend_marks(HMGL gr, int num);
 /*****************************************************************************/
 /*		1D plotting functions												 */
 /*****************************************************************************/
@@ -265,8 +290,6 @@ void mgl_fplot_xyz(HMGL graph, const char *fx, const char *fy, const char *fz, c
 void mgl_plot_xyz(HMGL graph, const HMDT x, const HMDT y, const HMDT z, const char *pen);
 void mgl_plot_xy(HMGL graph, const HMDT x, const HMDT y, const char *pen);
 void mgl_plot(HMGL graph, const HMDT y, const char *pen);
-void mgl_plot_2(HMGL graph, const HMDT a, const char *pen);
-void mgl_plot_3(HMGL graph, const HMDT a, const char *pen);
 void mgl_radar(HMGL graph, const HMDT a, const char *pen, mreal r);
 void mgl_boxplot_xy(HMGL graph, const HMDT x, const HMDT a, const char *pen);
 void mgl_boxplot(HMGL graph, const HMDT a, const char *pen);
@@ -278,33 +301,24 @@ void mgl_area_xy(HMGL graph, const HMDT x, const HMDT y, const char *pen);
 void mgl_area_xys(HMGL graph, const HMDT x, const HMDT y, const char *pen);
 void mgl_area_s(HMGL graph, const HMDT y, const char *pen);
 void mgl_area(HMGL graph, const HMDT y, const char *pen);
-void mgl_area_2(HMGL graph, const HMDT a, const char *pen);
-void mgl_area_3(HMGL graph, const HMDT a, const char *pen);
 void mgl_region_xy(HMGL graph, const HMDT x, const HMDT y1, const HMDT y2, const char *pen, int inside);
 void mgl_region(HMGL graph, const HMDT y1, const HMDT y2, const char *pen, int inside);
 void mgl_mark(HMGL graph, mreal x,mreal y,mreal z,char mark);
 void mgl_stem_xyz(HMGL graph, const HMDT x, const HMDT y, const HMDT z, const char *pen);
 void mgl_stem_xy(HMGL graph, const HMDT x, const HMDT y, const char *pen);
 void mgl_stem(HMGL graph, const HMDT y,	const char *pen);
-void mgl_stem_2(HMGL graph, const HMDT a, const char *pen);
-void mgl_stem_3(HMGL graph, const HMDT a, const char *pen);
 void mgl_step_xyz(HMGL graph, const HMDT x, const HMDT y, const HMDT z, const char *pen);
 void mgl_step_xy(HMGL graph, const HMDT x, const HMDT y, const char *pen);
 void mgl_step(HMGL graph, const HMDT y,	const char *pen);
-void mgl_step_2(HMGL graph, const HMDT a, const char *pen);
-void mgl_step_3(HMGL graph, const HMDT a, const char *pen);
 void mgl_bars_xyz(HMGL graph, const HMDT x, const HMDT y, const HMDT z, const char *pen);
 void mgl_bars_xy(HMGL graph, const HMDT x, const HMDT y, const char *pen);
 void mgl_bars(HMGL graph, const HMDT y,	const char *pen);
-void mgl_bars_2(HMGL graph, const HMDT a, const char *pen);
-void mgl_bars_3(HMGL graph, const HMDT a, const char *pen);
 void mgl_barh_yx(HMGL graph, const HMDT y, const HMDT v, const char *pen);
 void mgl_barh(HMGL graph, const HMDT v,	const char *pen);
 /*****************************************************************************/
 /*		Advanced 1D plotting functions												 */
 /*****************************************************************************/
 void mgl_torus(HMGL graph, const HMDT r, const HMDT z, const char *pen);
-void mgl_torus_2(HMGL graph, const HMDT a, const char *pen);
 void mgl_text_xyz(HMGL graph, const HMDT x, const HMDT y, const HMDT z,const char *text, const char *font, mreal size);
 void mgl_text_xy(HMGL graph, const HMDT x, const HMDT y, const char *text, const char *font, mreal size);
 void mgl_text_y(HMGL graph, const HMDT y, const char *text, const char *font, mreal size);
@@ -422,6 +436,10 @@ void mgl_pipe_xyz(HMGL graph, const HMDT x, const HMDT y, const HMDT z, const HM
 void mgl_pipe_3d(HMGL graph, const HMDT ax, const HMDT ay, const HMDT az, const char *sch, mreal r0, int num, int central);
 void mgl_dew_xy(HMGL gr, const HMDT x, const HMDT y, const HMDT ax, const HMDT ay, const char *sch,mreal zVal);
 void mgl_dew_2d(HMGL gr, const HMDT ax, const HMDT ay, const char *sch,mreal zVal);
+
+void mgl_grad_xyz(HMGL graph, const HMDT x, const HMDT y, const HMDT z, const HMDT ph, const char *sch, int num);
+void mgl_grad_xy(HMGL graph, const HMDT x, const HMDT y, const HMDT ph, const char *sch, int num, mreal zVal);
+void mgl_grad(HMGL graph, const HMDT ph, const char *sch, int num, mreal zVal);
 /*****************************************************************************/
 /*		3D plotting functions												 */
 /*****************************************************************************/
@@ -461,6 +479,13 @@ void mgl_beam(HMGL graph, const HMDT tr, const HMDT g1, const HMDT g2, const HMD
 void mgl_triplot_xyzc(HMGL gr, const HMDT nums, const HMDT x, const HMDT y, const HMDT z, const HMDT c, const char *sch);
 void mgl_triplot_xyz(HMGL gr, const HMDT nums, const HMDT x, const HMDT y, const HMDT z, const char *sch);
 void mgl_triplot_xy(HMGL gr, const HMDT nums, const HMDT x, const HMDT y, const char *sch, mreal zVal);
+void mgl_quadplot_xyzc(HMGL gr, const HMDT nums, const HMDT x, const HMDT y, const HMDT z, const HMDT c, const char *sch);
+void mgl_quadplot_xyz(HMGL gr, const HMDT nums, const HMDT x, const HMDT y, const HMDT z, const char *sch);
+void mgl_quadplot_xy(HMGL gr, const HMDT nums, const HMDT x, const HMDT y, const char *sch, mreal zVal);
+void mgl_tricont_xyzcv(HMGL gr, const HMDT v, const HMDT nums, const HMDT x, const HMDT y, const HMDT z, const HMDT c, const char *sch, mreal zVal);
+void mgl_tricont_xyzv(HMGL gr, const HMDT v, const HMDT nums, const HMDT x, const HMDT y, const HMDT z, const char *sch, mreal zVal);
+void mgl_tricont_xyzc(HMGL gr, const HMDT nums, const HMDT x, const HMDT y, const HMDT z, const HMDT c, const char *sch, int n, mreal zVal);
+void mgl_tricont_xyz(HMGL gr, const HMDT nums, const HMDT x, const HMDT y, const HMDT z, const char *sch, int n, mreal zVal);
 void mgl_dots(HMGL gr, const HMDT x, const HMDT y, const HMDT z, const char *sch);
 void mgl_dots_a(HMGL gr, const HMDT x, const HMDT y, const HMDT z, const HMDT a, const char *sch);
 void mgl_dots_tr(HMGL gr, const HMDT tr, const char *sch);
@@ -557,7 +582,14 @@ void mgl_data_diff(HMDT dat, const char *dir);
 void mgl_data_diff_par(HMDT dat, const HMDT v1, const HMDT v2, const HMDT v3);
 void mgl_data_diff2(HMDT dat, const char *dir);
 void mgl_data_swap(HMDT dat, const char *dir);
+void mgl_data_roll(HMDT dat, char dir, int num);
 void mgl_data_mirror(HMDT dat, const char *dir);
+
+void mgl_data_hankel(HMDT dat, const char *dir);
+void mgl_data_sinfft(HMDT dat, const char *dir);
+void mgl_data_cosfft(HMDT dat, const char *dir);
+void mgl_data_fill_sample(HMDT dat, int num, const char *how);
+
 mreal mgl_data_spline(const HMDT dat, mreal x,mreal y,mreal z);
 mreal mgl_data_spline1(const HMDT dat, mreal x,mreal y,mreal z);
 mreal mgl_data_linear(const HMDT dat, mreal x,mreal y,mreal z);
