@@ -57,6 +57,22 @@ void mgl_calc_xyz_(uintptr_t *gr, int *xs, int *ys, mreal *x, mreal *y, mreal *z
 void mgl_calc_scr_(uintptr_t *gr, mreal *x, mreal *y, mreal *z, int *xs, int *ys);
 //void mgl_fltk_thread_();
 //void mgl_qt_thread_();
+
+void mgl_wnd_set_delay_(uintptr_t *gr, mreal *dt);
+void mgl_wnd_set_auto_clf_(uintptr_t *gr, int *val);
+void mgl_wnd_set_show_mouse_pos_(uintptr_t *gr, int *val);
+void mgl_wnd_set_clf_update_(uintptr_t *gr, int *val);
+void mgl_wnd_toggle_alpha_(uintptr_t *gr);
+void mgl_wnd_toggle_light_(uintptr_t *gr);
+void mgl_wnd_toggle_zoom_(uintptr_t *gr);
+void mgl_wnd_toggle_rotate_(uintptr_t *gr);
+void mgl_wnd_toggle_no_(uintptr_t *gr);
+void mgl_wnd_update_(uintptr_t *gr);
+void mgl_wnd_reload_(uintptr_t *gr, int *o);
+void mgl_wnd_adjust_(uintptr_t *gr);
+void mgl_wnd_next_frame_(uintptr_t *gr);
+void mgl_wnd_prev_frame_(uintptr_t *gr);
+void mgl_wnd_animation_(uintptr_t *gr);
 /*****************************************************************************/
 uintptr_t mgl_create_data_();
 uintptr_t mgl_create_data_size_(int *nx, int *ny, int *nz);
@@ -91,6 +107,7 @@ void mgl_set_scheme_(uintptr_t *gr, char *sch, int);
 void mgl_load_font_(uintptr_t *gr, char *name, char *path, int l, int n);
 void mgl_copy_font_(uintptr_t *gr, uintptr_t *gr_from);
 void mgl_restore_font_(uintptr_t *gr);
+int mgl_get_warn_(uintptr_t *gr);
 /*****************************************************************************/
 /*		Export to file or to memory														 */
 /*****************************************************************************/
@@ -130,15 +147,19 @@ void mgl_set_ambbr_(uintptr_t *gr, mreal *i);
 /*****************************************************************************/
 /*		Scale and rotate													 */
 /*****************************************************************************/
+void mgl_mat_push_(uintptr_t *gr);
+void mgl_mat_pop_(uintptr_t *gr);
 void mgl_identity_(uintptr_t *graph, int *rel);
 void mgl_clf_(uintptr_t *graph);
 void mgl_flush_(uintptr_t *gr);
 void mgl_clf_rgb_(uintptr_t *graph, mreal *r, mreal *g, mreal *b);
 void mgl_subplot_(uintptr_t *graph, int *nx,int *ny,int *m);
 void mgl_subplot_d_(uintptr_t *graph, int *nx,int *ny,int *m,mreal *dx,mreal *dy);
+void mgl_subplot_s_(uintptr_t *graph, int *nx,int *ny,int *m, const char *s,int);
 void mgl_inplot_(uintptr_t *graph, mreal *x1,mreal *x2,mreal *y1,mreal *y2);
 void mgl_relplot_(uintptr_t *graph, mreal *x1,mreal *x2,mreal *y1,mreal *y2);
 void mgl_columnplot_(uintptr_t *graph, int *num, int *i);
+void mgl_columnplot_d_(uintptr_t *graph, int *num, int *i, mreal *d);
 void mgl_stickplot_(uintptr_t *graph, int *num, int *i, mreal *tet, mreal *phi);
 void mgl_aspect_(uintptr_t *graph, mreal *Ax,mreal *Ay,mreal *Az);
 void mgl_rotate_(uintptr_t *graph, mreal *TetX,mreal *TetZ,mreal *TetY);
@@ -156,6 +177,8 @@ void mgl_set_caxis_(uintptr_t *graph, mreal *C1,mreal *C2);
 void mgl_set_axis_(uintptr_t *graph, mreal *x1, mreal *y1, mreal *z1, mreal *x2, mreal *y2, mreal *z2, mreal *x0, mreal *y0, mreal *z0);
 void mgl_set_axis_3d_(uintptr_t *graph, mreal *x1, mreal *y1, mreal *z1, mreal *x2, mreal *y2, mreal *z2);
 void mgl_set_axis_2d_(uintptr_t *graph, mreal *x1, mreal *y1, mreal *x2, mreal *y2);
+inline void mgl_set_ranges_(uintptr_t *graph, mreal *x1, mreal *x2, mreal *y1, mreal *y2, mreal *z1, mreal *z2)
+{	mgl_set_axis_3d_(graph, x1,y1,z1,x2,y2,z2);	};
 void mgl_set_origin_(uintptr_t *graph, mreal *x0, mreal *y0, mreal *z0);
 void mgl_set_tick_origin_(uintptr_t *graph, mreal *x0, mreal *y0, mreal *z0);
 void mgl_set_crange_(uintptr_t *graph, uintptr_t *a, int *add);
@@ -208,6 +231,7 @@ void mgl_clear_legend_(uintptr_t *graph);
 void mgl_legend_xy_(uintptr_t *graph, mreal *x, mreal *y, const char *font, mreal *size, mreal *llen,int);
 void mgl_legend_(uintptr_t *graph, int *where, const char *font, mreal *size, mreal *llen,int);
 void mgl_set_legend_box_(uintptr_t *gr, int *enable);
+void mgl_set_legend_marks_(uintptr_t *gr, int *num);
 /*****************************************************************************/
 /*		1D plotting functions												 */
 /*****************************************************************************/
@@ -216,8 +240,6 @@ void mgl_fplot_xyz_(uintptr_t *graph, const char *fx, const char *fy, const char
 void mgl_plot_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *pen,int);
 void mgl_plot_xy_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, const char *pen,int);
 void mgl_plot_(uintptr_t *graph, uintptr_t *y,	const char *pen,int);
-void mgl_plot_2_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
-void mgl_plot_3_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
 void mgl_radar_(uintptr_t *graph, uintptr_t *a,	const char *pen, mreal *r, int);
 void mgl_boxplot_xy_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, const char *pen,int);
 void mgl_boxplot_(uintptr_t *graph, uintptr_t *y, const char *pen,int);
@@ -228,33 +250,24 @@ void mgl_area_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, c
 void mgl_area_xy_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, const char *pen,int);
 void mgl_area_s_(uintptr_t *graph, uintptr_t *y, const char *pen,int);
 void mgl_area_(uintptr_t *graph, uintptr_t *y, const char *pen,int);
-void mgl_area_2_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
-void mgl_area_3_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
 void mgl_region_xy_(uintptr_t *graph, uintptr_t *x, uintptr_t *y1, uintptr_t *y2, const char *pen, int *inside, int);
 void mgl_region_(uintptr_t *graph, uintptr_t *y1, uintptr_t *y2, const char *pen, int *inside, int);
 void mgl_mark_(uintptr_t *graph, mreal *x,mreal *y,mreal *z,const char *mark,int);
 void mgl_stem_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *pen,int);
 void mgl_stem_xy_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, const char *pen,int);
 void mgl_stem_(uintptr_t *graph, uintptr_t *y,	const char *pen,int);
-void mgl_stem_2_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
-void mgl_stem_3_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
 void mgl_step_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *pen,int);
 void mgl_step_xy_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, const char *pen,int);
 void mgl_step_(uintptr_t *graph, uintptr_t *y,	const char *pen,int);
-void mgl_step_2_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
-void mgl_step_3_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
 void mgl_bars_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *pen,int);
 void mgl_bars_xy_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, const char *pen,int);
 void mgl_bars_(uintptr_t *graph, uintptr_t *y,	const char *pen,int);
-void mgl_bars_2_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
-void mgl_bars_3_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
 void mgl_barh_yx_(uintptr_t *graph, uintptr_t *y, uintptr_t *v, const char *pen,int);
 void mgl_barh_(uintptr_t *graph, uintptr_t *v,	const char *pen,int);
 /*****************************************************************************/
 /*		Advanced 1D plotting functions												 */
 /*****************************************************************************/
 void mgl_torus_(uintptr_t *graph, uintptr_t *r, uintptr_t *z, const char *pen,int);
-void mgl_torus_2_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
 void mgl_text_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z,const char *text, const char *font, mreal *size,int,int);
 void mgl_text_xy_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, const char *text, const char *font, mreal *size,int,int);
 void mgl_text_y_(uintptr_t *graph, uintptr_t *y, const char *text, const char *font, mreal *size,int,int);
@@ -371,6 +384,10 @@ void mgl_pipe_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, u
 void mgl_pipe_3d_(uintptr_t *graph, uintptr_t *ax, uintptr_t *ay, uintptr_t *az, const char *sch, mreal *r0, int *num, int *central,int);
 void mgl_dew_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *ax, uintptr_t *ay, const char *sch, mreal *zVal,int l);
 void mgl_dew_2d_(uintptr_t *gr, uintptr_t *ax, uintptr_t *ay, const char *sch, mreal *zVal,int l);
+
+void mgl_grad_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *ph, const char *sch, int *num, int);
+void mgl_grad_xy_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *ph, const char *sch, int *num, mreal *zVal,int);
+void mgl_grad_(uintptr_t *graph, uintptr_t *ph, const char *sch, int *num, mreal *zVal,int);
 /*****************************************************************************/
 /*		3D plotting functions												 */
 /*****************************************************************************/
@@ -420,6 +437,15 @@ void mgl_beam_(uintptr_t *gr, uintptr_t *tr, uintptr_t *g1, uintptr_t *g2, uintp
 void mgl_triplot_xyzc_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *c, const char *sch,int);
 void mgl_triplot_xyz_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *sch,int);
 void mgl_triplot_xy_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, uintptr_t *y, const char *sch, mreal *zVal,int);
+void mgl_quadplot_xyzc_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *c, const char *sch,int);
+void mgl_quadplot_xyz_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *sch,int);
+void mgl_quadplot_xy_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, uintptr_t *y, const char *sch, mreal *zVal,int);
+
+void mgl_tricont_xyzcv_(uintptr_t *gr, uintptr_t *v, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *c, const char *sch, mreal *zVal,int);
+void mgl_tricont_xyzv_(uintptr_t *gr, uintptr_t *v, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *sch, mreal *zVal,int);
+void mgl_tricont_xyzc_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *c, const char *sch, int *n, mreal *zVal, int);
+void mgl_tricont_xyz_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *sch, int *n, mreal *zVal, int);
+
 void mgl_dots_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *sch,int);
 void mgl_dots_a_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, const char *sch,int);
 void mgl_dots_tr_(uintptr_t *gr, uintptr_t *tr, const char *sch,int);
@@ -514,7 +540,14 @@ void mgl_data_diff_(uintptr_t *dat, const char *dir,int);
 void mgl_data_diff_par_(uintptr_t *dat, uintptr_t *v1, uintptr_t *v2, uintptr_t *v3);
 void mgl_data_diff2_(uintptr_t *dat, const char *dir,int);
 void mgl_data_swap_(uintptr_t *dat, const char *dir,int);
+void mgl_data_roll_(uintptr_t *dat, const char *dir, int *num, int);
 void mgl_data_mirror_(uintptr_t *dat, const char *dir,int);
+
+void mgl_data_hankel_(uintptr_t *dat, const char *dir,int);
+void mgl_data_sinfft_(uintptr_t *dat, const char *dir,int);
+void mgl_data_cosfft_(uintptr_t *dat, const char *dir,int);
+void mgl_data_fill_sample_(uintptr_t *dat, int num, const char *how,int);
+
 mreal mgl_data_spline_(uintptr_t *dat, mreal *x,mreal *y,mreal *z);
 mreal mgl_data_spline1_(uintptr_t *dat, mreal *x,mreal *y,mreal *z);
 mreal mgl_data_linear_(uintptr_t *dat, mreal *x,mreal *y,mreal *z);
