@@ -47,7 +47,7 @@ struct mglCommand
 	/// Function for exporting in C++ (can be NULL)
 	void (*save)(wchar_t out[1024], long n, mglArg *a, int k[10]);
 	bool create;	///< Should parser create 1st the array automatically
-	int type;		///< Type of command: 0 - data plot, 1 - other plot, 
+	int type;		///< Type of command: 0 - data plot, 1 - other plot,
 					///	2 - setup, 3 - data handle, 4 - subplot, 5 - program
 };
 extern mglCommand mgls_base_cmd[];
@@ -64,7 +64,7 @@ struct mglVar
 	void (*func)(void *);	///< Callback function for destroying
 
 	mglVar()	{	o=0;	s[0]=0;		next=prev=0;	func=0;	temp=false;	};
-	~mglVar();
+	virtual ~mglVar();
 	/// Move variable after \a var and copy \a func from \a var (if \a func is 0)
 	void MoveAfter(mglVar *var);
 };
@@ -77,7 +77,7 @@ struct mglNum
 	mglNum *next;	///< Pointer to next instance in list
 	mglNum *prev;	///< Pointer to prev instance in list
 	mglNum()	{	d=0;	s[0]=0;		next=prev=0;	};
-	~mglNum();
+	virtual ~mglNum();
 	/// Move variable after \a var and copy \a func from \a var (if \a func is 0)
 	void MoveAfter(mglNum *var);
 };
@@ -114,7 +114,7 @@ public:
 	wchar_t *op1, *op2;	///< Buffer for options (are used if out!=NULL)
 
 	mglParse(bool setsize=false);
-	~mglParse();
+	~mglParse();	///< NOTE: virtual destructor here give SegFault under GCC 4.5.2
 	/// Find the command by the keyword name
 	mglCommand *FindCommand(const wchar_t *name, bool prog=false);
 	/// Parse and execute the string of MGL script
@@ -166,25 +166,25 @@ public:
 	/// Delete variable by its name
 	void DeleteVar(const wchar_t *name);
 private:
-	long parlen;	///< Length of parameter strings
+	long parlen;		///< Length of parameter strings
 	wchar_t *par[40];	///< Parameter for substituting instead of $1, ..., $9
 	wchar_t *out;		///< Buffer for writing C++ code (if not NULL)
 	wchar_t leg[128];	///< Buffer for legend
-	bool opt[16];	///< Set on/off optional parameters for command argument
-	mreal val[20];	///< Values for optional parameters
-	bool Once;		///< Flag for command which should be executed only once
-	bool Skip;		///< Flag that commands should be skiped (inside 'once' block)
+	bool opt[16];		///< Set on/off optional parameters for command argument
+	mreal val[20];		///< Values for optional parameters
+	bool Once;			///< Flag for command which should be executed only once
+	bool Skip;			///< Flag that commands should be skiped (inside 'once' block)
 	int if_stack[40];	///< Stack for if-else-endif commands
-	int if_pos;		///< position in if_stack
-	mglFunc *func;	///< function names and position
+	int if_pos;			///< position in if_stack
+	mglFunc *func;		///< function names and position
 	mglFnStack *fn_stack;	///< function calls stack
-	int fn_pos;		///< position in function stack
-	int fn_num;		///< size of function stack
-	int if_for[40];	///< position in if_stack for for-cycle start
-	mglData *fval;	///< Values for for-cycle. Note that nx - number of elements, ny - next element, nz - address (or string number) of first cycle command
+	int fn_pos;			///< position in function stack
+	int fn_num;			///< size of function stack
+	int if_for[40];		///< position in if_stack for for-cycle start
+	mglData *fval;		///< Values for for-cycle. Note that nx - number of elements, ny - next element, nz - address (or string number) of first cycle command
 	int for_stack[40];	///< The order of for-variables
-	int for_addr;	///< Flag for saving address in variable (for_addr-1)
-	bool for_br;	///< Break is switched on (skip all comands until 'next')
+	int for_addr;		///< Flag for saving address in variable (for_addr-1)
+	bool for_br;		///< Break is switched on (skip all comands until 'next')
 
 	/// Parse command
 	int Exec(mglGraph *gr, const wchar_t *com, long n, mglArg *a, const wchar_t *var);
