@@ -83,7 +83,7 @@ public:
 	mglData(const HMDT aa, bool copy=true)
 	{	if(copy)	{	a=mgl_create_data();	if(aa)	mgl_data_set(a,aa);	}	else	a=aa;	del=copy;	};
 	mglData(const char *fname)	{	a=mgl_create_data_file(fname);	del=true;	}
-	~mglData()	{	if(a && del)	mgl_delete_data(a);	};
+	virtual ~mglData()	{	if(a && del)	mgl_delete_data(a);	};
 	mglData(const mglData &d)	{	a=mgl_create_data();	mgl_data_set(a,d.a);	del=true;	};
 
 	mglData(int size, const float *d)
@@ -290,7 +290,7 @@ public:
 		else if(kind==3)	self=mgl_create_graph_idtf();
 		else		self=mgl_create_graph_zb(width, height);
 	};
-	~mglGraph()	{	mgl_delete_graph(self);	};
+	virtual ~mglGraph()	{	mgl_delete_graph(self);	};
 	inline HMGL Self()	{	return self;	};
 
 	inline int  GetWarn()	{	return mgl_get_warn(self);	};
@@ -329,7 +329,7 @@ public:
 	inline void CopyFont(mglGraph *gr)		{	mgl_copy_font(self, gr->Self());};
 	inline void RestoreFont()				{	mgl_restore_font(self);	};
 
-	inline void ShowImage(const char *viewer=0, bool keep=0)
+	inline void ShowImage(const char *viewer, bool keep=0)
 	{	mgl_show_image(self, viewer, keep);	};
 	inline void WriteFrame(const char *fname,const char *descr="")
 	{	mgl_write_frame(self, fname, descr);	};
