@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -28,24 +28,24 @@
 //-----------------------------------------------------------------------------
 void mglGraph::Surf(const char *eqZ, const char *sch, int n)
 {
-	// TODO Add strong function variation analisys
+	// TODO Add strong function variation analisys ???
 	if(eqZ==0 || eqZ[0]==0)	return;		// nothing to plot
+	if(n<=0)	n=100;
 	mglData z(n,n);
 	mglFormula *eq = new mglFormula(eqZ);
 	register int i,j;
 	mreal dx = (Max.x - Min.x)/(n-1.), dy = (Max.y - Min.y)/(n-1.);
 	for(i=0;i<n;i++)	for(j=0;j<n;j++)
-	{
 		z.a[i+n*j] = eq->Calc(Min.x+i*dx, Min.y+j*dy);
-	}
 	Surf(z, sch);
 	delete eq;
 }
 //-----------------------------------------------------------------------------
 void mglGraph::Surf(const char *eqX, const char *eqY, const char *eqZ, const char *sch, int n)
 {
-	// TODO Add strong function variation analisys
+	// TODO Add strong function variation analisys ???
 	mglData x(n,n), y(n,n), z(n,n);
+	if(n<=0)	n=100;
 	mglFormula *ex, *ey, *ez;
 	ex = new mglFormula(eqX ? eqX : "u");
 	ey = new mglFormula(eqY ? eqY : "v");
@@ -67,7 +67,7 @@ void mglGraph::Surf(const char *eqX, const char *eqY, const char *eqZ, const cha
 //	Primitives
 //
 //-----------------------------------------------------------------------------
-void mglGraph::Face(mglPoint p1, mglPoint p2, mglPoint p3, mglPoint p4, const char *stl, int n)
+void mglGraph::Face(mglPoint p1, mglPoint p2, mglPoint p3, mglPoint p4, const char *stl, int n,mreal val)
 {
 	static int cgid=1;	StartGroup("Face",cgid++);
 	register long i,j,i0;
@@ -78,9 +78,10 @@ void mglGraph::Face(mglPoint p1, mglPoint p2, mglPoint p3, mglPoint p4, const ch
 	bool cut = Cut;	Cut = true;
 
 	SetScheme(stl);
-	if(NumCol)	c1 = cmap[0];
-	bool all = NumCol>3;
+	if(stl && *stl && NumCol)	c1 = cmap[0];
+	bool all = stl && strlen(stl)>4 && NumCol>4;
 	if(all)	{	c2=cmap[1];	c3=cmap[2];	c4=cmap[3];	}
+	else if(!isnan(val))	c1 = GetC(val);
 
 	for(i=0;i<n;i++)	for(j=0;j<n;j++)
 	{
@@ -144,7 +145,9 @@ void mglGraph::Drop(mglPoint p, mglPoint q, mreal r, mglColor c, mreal sh, mreal
 		ScalePoint(pp[3*i0+0],pp[3*i0+1],pp[3*i0+2]);
 	}
 	Cut = cut;
+	int mnt=mglNumThr;	mglNumThr=1;	// too many faces
 	surf_plot(n,n,pp,0,0);
+	mglNumThr = mnt;
 	EndGroup();
 	delete []pp;
 }
