@@ -157,28 +157,31 @@ void mgl_set_ambbr(HMGL gr, mreal i)
 //-----------------------------------------------------------------------------
 //		Scale and rotate
 //-----------------------------------------------------------------------------
+/// Push transformation matrix into stack.
+void mgl_mat_push(HMGL gr)	{	gr->Push();	}
+/// Pop transformation matrix into stack.
+void mgl_mat_pop(HMGL gr)	{	gr->Pop();	}
 /// Clear transformation matrix.
-void mgl_identity(HMGL gr, int rel)
-{	gr->Identity(rel);	}
+void mgl_identity(HMGL gr, int rel)	{	gr->Identity(rel);	}
 /// Clear up the frame
-void mgl_clf(HMGL gr)
-{	gr->Clf();	}
+void mgl_clf(HMGL gr)	{	gr->Clf();	}
 /// Clear up the frame
-void mgl_clf_rgb(HMGL gr, mreal r, mreal g, mreal b)
-{	gr->Clf(mglColor(r,g,b));	}
-/// Put further plotting in some region of whole frame surface.
-void mgl_subplot(HMGL gr, int nx,int ny,int m)
-{	gr->SubPlot(nx,ny,m);	}
+void mgl_clf_rgb(HMGL gr, mreal r, mreal g, mreal b){	gr->Clf(mglColor(r,g,b));	}
 /// Put further plotting in some region of whole frame surface.
+void mgl_subplot(HMGL gr, int nx,int ny,int m)		{	gr->SubPlot(nx,ny,m);	}
 void mgl_subplot_d(HMGL gr, int nx,int ny,int m,mreal dx,mreal dy)
 {	gr->SubPlot(nx,ny,m,dx,dy);	}
+void mgl_subplot_s(HMGL gr, int nx,int ny,int m,const char *style)
+{	gr->SubPlot(nx,ny,m,style);	}
 /// Put further plotting in some region of whole frame surface.
 void mgl_inplot(HMGL gr, mreal x1,mreal x2,mreal y1,mreal y2)
-{	gr->InPlot(x1,x2,y1,y2);	}
+{	gr->InPlot(x1,x2,y1,y2,false);	}
 void mgl_relplot(HMGL gr, mreal x1,mreal x2,mreal y1,mreal y2)
 {	gr->InPlot(x1,x2,y1,y2,true);	}
 void mgl_columnplot(HMGL gr, int num, int i)
 {	gr->ColumnPlot(num,i);	}
+void mgl_columnplot_d(HMGL gr, int num, int i, mreal d)
+{	gr->ColumnPlot(num,i,d);	}
 void mgl_stickplot(HMGL gr, int num, int i, mreal tet, mreal phi)
 {	gr->StickPlot(num, i, tet, phi);	}
 /// Set aspect ratio for further plotting.
@@ -341,6 +344,8 @@ void mgl_legend(HMGL gr, int where, const char *font, mreal size, mreal llen)
 {	if(gr)	gr->Legend(where, font, size, llen);	}
 void mgl_set_legend_box(HMGL gr, int enable)
 {	gr->LegendBox = enable;	}
+void mgl_set_legend_marks(HMGL gr, int num)
+{	gr->SetLegendMarks(num);	}
 //-----------------------------------------------------------------------------
 const unsigned char *mgl_get_rgb(HMGL graph)
 {
@@ -393,6 +398,37 @@ void mgl_calc_scr(HMGL gr, mreal x, mreal y, mreal z, int *xs, int *ys)
 	if(g && xs && ys)	g->CalcScr(mglPoint(x,y,z),xs,ys);
 }
 //-----------------------------------------------------------------------------
+void mgl_wnd_set_delay(HMGL gr, mreal dt)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->Delay = dt;	}
+void mgl_wnd_set_auto_clf(HMGL gr, int val)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->AutoClf = val;	}
+void mgl_wnd_set_show_mouse_pos(HMGL gr, int val)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->ShowMousePos = val;	}
+void mgl_wnd_set_clf_update(HMGL gr, int val)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->ClfOnUpdate = val;	}
+void mgl_wnd_toggle_alpha(HMGL gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->ToggleAlpha();	}
+void mgl_wnd_toggle_light(HMGL gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->ToggleLight();	}
+void mgl_wnd_toggle_zoom(HMGL gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->ToggleZoom();	}
+void mgl_wnd_toggle_rotate(HMGL gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->ToggleRotate();	}
+void mgl_wnd_toggle_no(HMGL gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->ToggleNo();	}
+void mgl_wnd_update(HMGL gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->Update();	}
+void mgl_wnd_reload(HMGL gr, int o)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->ReLoad(o);	}
+void mgl_wnd_adjust(HMGL gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->Adjust();	}
+void mgl_wnd_next_frame(HMGL gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->NextFrame();	}
+void mgl_wnd_prev_frame(HMGL gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->PrevFrame();	}
+void mgl_wnd_animation(HMGL gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);	if(g)	g->Animation();	}
+//-----------------------------------------------------------------------------
 mreal mgl_data_get_value(const HMDT d, int i, int j, int k)
 {	return	d->a[i+d->nx*(j+d->ny*k)];	}
 void mgl_data_set_value(HMDT d, mreal v, int i, int j, int k)
@@ -450,4 +486,5 @@ void mgl_set_tick_len(HMGL gr, mreal len, mreal stt)
 {	gr->SetTickLen(len,stt);	}
 void mgl_set_tick_stl(HMGL gr, const char *stl, const char *sub)
 {	gr->SetTickStl(stl,sub);	}
+int mgl_get_warn(HMGL gr)	{	return gr->WarnCode;	}
 //-----------------------------------------------------------------------------
