@@ -96,15 +96,17 @@ mreal mgl_data_min(const HMDT d)	{	return d->Minimal();	}
 mreal *mgl_data_value(HMDT d, int i,int j,int k)
 {	return d->a+i+d->nx*(j+d->ny*k);	}
 /// Swap left and right part of the data in given direction (useful for fourier spectrums)
-void mgl_data_swap(HMDT d, const char *dir)
-{	d->Swap(dir);	}
+void mgl_data_swap(HMDT d, const char *dir)		{	d->Swap(dir);	}
+/// Swap left and right part of the data in given direction (useful for fourier spectrums)
+void mgl_data_roll(HMDT d, char dir, int num)	{	d->Roll(dir,num);	}
 /// Mirror the data in given direction
-void mgl_data_mirror(HMDT d, const char *dir)
-{	d->Mirror(dir);	}
-void mgl_data_insert(HMDT dat, char dir, int at, int num)
-{	dat->Insert(dir,at,num);	}
-void mgl_data_delete(HMDT dat, char dir, int at, int num)
-{	dat->Delete(dir,at,num);	}
+void mgl_data_mirror(HMDT d, const char *dir)	{	d->Mirror(dir);	}
+void mgl_data_fill_sample(HMDT d, int num, const char *how)	{	d->FillSample(num,how);	}
+void mgl_data_insert(HMDT d, char dir, int at, int num){	d->Insert(dir,at,num);	}
+void mgl_data_delete(HMDT d, char dir, int at, int num){	d->Delete(dir,at,num);	}
+void mgl_data_hankel(HMDT d, const char *dir)	{	d->Hankel(dir);	}
+void mgl_data_sinfft(HMDT d, const char *dir)	{	d->SinFFT(dir);	}
+void mgl_data_cosfft(HMDT d, const char *dir)	{	d->CosFFT(dir);	}
 //-----------------------------------------------------------------------------
 /// Allocate memory and copy the data from the (mreal *) array
 void mgl_data_set_float(HMDT d, const float *A,int NX,int NY,int NZ)
@@ -247,6 +249,8 @@ void mgl_data_swap_(uintptr_t *d, const char *dir,int l)
 	_DT_->Swap(s);
 	delete []s;
 }
+void mgl_data_roll_(uintptr_t *d, const char *dir, int *num, int)
+{	_DT_->Roll(*dir, *num);	}
 /// Mirror the data in given direction
 void mgl_data_mirror_(uintptr_t *d, const char *dir,int l)
 {
@@ -254,10 +258,31 @@ void mgl_data_mirror_(uintptr_t *d, const char *dir,int l)
 	_DT_->Mirror(s);
 	delete []s;
 }
+void mgl_data_fill_sample_(uintptr_t *d, int num, const char *how,int l)
+{
+	char *s=new char[l+1];	memcpy(s,how,l);	s[l]=0;
+	_DT_->FillSample(num,s);
+	delete []s;
+}
 void mgl_data_insert_(uintptr_t *d, const char *dir, int *at, int *num, int)
 {	_DT_->Insert(*dir,*at,*num);	}
 void mgl_data_delete_(uintptr_t *d, const char *dir, int *at, int *num, int)
 {	_DT_->Delete(*dir,*at,*num);	}
+void mgl_data_hankel_(uintptr_t *d, const char *dir,int l)
+{
+	char *s=new char[l+1];	memcpy(s,dir,l);	s[l]=0;
+	_DT_->Hankel(s);	delete []s;
+}
+void mgl_data_sinfft_(uintptr_t *d, const char *dir,int l)
+{
+	char *s=new char[l+1];	memcpy(s,dir,l);	s[l]=0;
+	_DT_->SinFFT(s);	delete []s;
+}
+void mgl_data_cosfft_(uintptr_t *d, const char *dir,int l)
+{
+	char *s=new char[l+1];	memcpy(s,dir,l);	s[l]=0;
+	_DT_->CosFFT(s);	delete []s;
+}
 //-----------------------------------------------------------------------------
 /// Allocate memory and copy the data from the (mreal *) array
 void mgl_data_set_float1_(uintptr_t *d, const float *A,int *NX)
