@@ -23,6 +23,7 @@
 #include <string.h>
 #ifndef NO_GSL
 #include <gsl/gsl_sf.h>
+#include <gsl/gsl_rng.h>
 #include <gsl/gsl_errno.h>
 #endif
 #include "mgl/mgl_eval.h"
@@ -75,6 +76,7 @@ EQ_LN,		// logarithm of x, ln(x)
 EQ_LG,		// decimal logarithm of x, lg(x) = ln(x)/ln(10)
 EQ_SIGN,	// sign of number
 EQ_STEP,	// step function
+EQ_INT,		// integer part [x]
 EQ_ABS,		// absolute value of x
 // special functions of 1 argument
 EQ_LI2,		// dilogarithm for a real argument Li2(x) = - \Re \int_0^x ds \log(1-s)/s.
@@ -118,63 +120,40 @@ EQ_EN,
 EQ_CL,		// Clausen function
 };
 //-----------------------------------------------------------------------------
-#define IM1 2147483563
-#define IM2 2147483399
-#define AM (1.0/IM1)
-#define IA1 40014
-#define IA2 40692
-#define IQ1 53668
-#define IQ2 52774
-#define IR1 12211
-#define IR2 3791
-#define NTAB 32
-#define NDIV (1+(IM1-1)/NTAB)
-#define RNMX (1.0 - 1.2e-7)
 #ifndef M_PI
 #define M_PI       3.14159265358979323846
 #endif
 //-----------------------------------------------------------------------------
-static long mgl_idum=0;
-void mgl_srnd(long seed)	{	mgl_idum = -abs(seed);	}
-//-----------------------------------------------------------------------------
+#ifndef NO_GSL
+gsl_rng *mgl_rng=0;
+#endif
+void mgl_srnd(long seed)
+{
+#ifndef NO_GSL
+	if(mgl_rng==0)
+	{
+		gsl_rng_env_setup();
+		mgl_rng = gsl_rng_alloc(gsl_rng_default);
+	}
+	gsl_rng_set(mgl_rng, seed);
+#else
+	srand(seed);
+#endif
+}
 double mgl_rnd()
 {
-	long &idum=mgl_idum;
-	long j;
-	long k;
-	static long idum2=123456789;
-	static long iy=0;
-	static long iv[NTAB];
-	double temp;
-	if(mgl_idum==0)
-		mgl_idum = -(long)(time(NULL));
-	if (mgl_idum <= 0) { 				// Initialize.
-		if (-(idum) < 1) idum=1;	// Be sure to prevent idum = 0.
-		else idum = -(idum);
-		idum2=(idum);
-		for (j=NTAB+7;j>=0;j--) {	// Load the shuffle table (after 8 warm-ups).
-			k=(idum)/IQ1;
-			idum=IA1*(idum-k*IQ1)-k*IR1;
-			if (idum < 0) idum += IM1;
-			if (j < NTAB) iv[j] = idum;
-		}
-		iy=iv[0];
+#ifndef NO_GSL
+	if(mgl_rng==0)
+	{
+		gsl_rng_env_setup();
+		mgl_rng = gsl_rng_alloc(gsl_rng_default);
+		gsl_rng_set(mgl_rng, time(0));
 	}
-	k=(idum)/IQ1; 					// Start here when not initializing.
-	idum=IA1*(idum-k*IQ1)-k*IR1;	// Compute idum=(IA1*idum) % IM1 without
-									// over ows by Schrage's method.
-	if (idum < 0) idum += IM1;
-	k=idum2/IQ2;
-	idum2=IA2*(idum2-k*IQ2)-k*IR2;	// Compute idum2=(IA2*idum) % IM2 likewise.
-	if (idum2 < 0) idum2 += IM2;
-	j=iy/NDIV; 						// Will be in the range 0..NTAB-1.
-	iy=iv[j]-idum2;					// Here idum is shufted, idum and idum2 are
-									// combined to generate output.
-	iv[j] = idum;
-	if (iy < 1) iy += IM1-1;
-	if ((temp=AM*iy) > RNMX)	// Because users don't expect endpoint values.
-		return RNMX;
-	else return temp;
+	return gsl_rng_uniform(mgl_rng);
+//	gsl_rng_free(r);
+#else
+	return rand()/(RAND_MAX-1.);
+#endif
 }
 //-----------------------------------------------------------------------------
 double mgl_ipow(double x,int n);
@@ -405,6 +384,7 @@ mglFormula::mglFormula(const char *string)
 		else if(!strcmp(name,"pow"))	Kod=EQ_POW;
 		else if(!strcmp(name,"mod"))	Kod=EQ_MOD;
 		else if(!strcmp(name,"i"))		Kod=EQ_BESI;
+		else if(!strcmp(name,"int"))	Kod=EQ_INT;
 		else if(!strcmp(name,"j"))		Kod=EQ_BESJ;
 		else if(!strcmp(name,"k"))		Kod=EQ_BESK;
 		else if(!strcmp(name,"y"))		Kod=EQ_BESY;
@@ -440,7 +420,8 @@ mreal mglFormula::Calc(mreal x,mreal y,mreal t,mreal u) const
 	a1['x'-'a'] = a1['r'-'a'] = x;
 	a1['y'-'a'] = a1['n'-'a'] = a1['v'-'a'] = y;
 	a1['z'-'a'] = a1['t'-'a'] = t;
-	return CalcIn(a1);
+	mreal a = CalcIn(a1);
+	return isfinite(a) ? a : NAN;
 }
 //-----------------------------------------------------------------------------
 // evaluate formula for 'x'='r', 'y'='n', 't'='z', 'u'='a', 'v'='b', 'w'='c' variables
@@ -454,21 +435,24 @@ mreal mglFormula::Calc(mreal x,mreal y,mreal t,mreal u,mreal v,mreal w) const
 	a1['x'-'a'] = a1['r'-'a'] = x;
 	a1['y'-'a'] = a1['n'-'a'] = y;
 	a1['z'-'a'] = a1['t'-'a'] = t;
-	return CalcIn(a1);
+	mreal a = CalcIn(a1);
+	return isfinite(a) ? a : NAN;
 }
 //-----------------------------------------------------------------------------
 // evaluate formula for arbitrary set of variables
 mreal mglFormula::Calc(const mreal var[MGL_VS]) const
 {
 	Error=0;
-	return CalcIn(var);
+	mreal a = CalcIn(var);
+	return isfinite(a) ? a : NAN;
 }
 //-----------------------------------------------------------------------------
 // evaluate derivate of formula respect to 'diff' variable for arbitrary set of other variables
 mreal mglFormula::CalcD(const mreal var[MGL_VS], char diff) const
 {
 	Error=0;
-	return CalcDIn(diff-'a', var);
+	mreal a = CalcDIn(diff-'a', var);
+	return isfinite(a) ? a : NAN;
 }
 //-----------------------------------------------------------------------------
 double cand(double a,double b)	{return a&&b?1:0;}
@@ -518,8 +502,8 @@ mreal mglFormula::CalcIn(const mreal *a1) const
 			,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2
 #endif
 		};
-	func_1 f1[41] = {sin,cos,tan,asin,acos,atan,sinh,cosh,tanh,
-					asinh,acosh,atanh,sqrt,exp,log,log10,sgn,stp,fabs
+	func_1 f1[42] = {sin,cos,tan,asin,acos,atan,sinh,cosh,tanh,
+					asinh,acosh,atanh,sqrt,exp,log,log10,sgn,stp,floor,fabs
 #ifndef NO_GSL
 			,gsl_sf_dilog,gslEllEc,gslEllFc,gslAi,gslBi,gsl_sf_erf,
 			gsl_sf_expint_3,gsl_sf_expint_Ei,gsl_sf_expint_E1,gsl_sf_expint_E2,
@@ -532,53 +516,45 @@ mreal mglFormula::CalcIn(const mreal *a1) const
 #endif
 		};
 //	if(Error)	return 0;
-	if(Kod==EQ_A)	return a1[(int)Res];
-	if(Kod==EQ_RND)	return mgl_rnd();
-	if(Kod==EQ_NUM) return Res;
-
+	if(Kod<EQ_LT)
+	{
+		if(Kod==EQ_RND)	return mgl_rnd();
+		else	return (Kod==EQ_A) ? a1[int(Res)] : Res;
+	}
 	double a = Left->CalcIn(a1);
-	if(isnan(a))	return NAN;
-
-	if(Kod<EQ_SIN)
+	if(!isnan(a))
 	{
-		double b = Right->CalcIn(a1);
-		if(isnan(b))	return NAN;
-		if(Kod==EQ_POW && ((a<0 && b>0) || (a<=0 && b<=0)))
-		{ Error=MGL_ERR_LOG; return NAN; }
-		if(Kod==EQ_LOG && (a<=0 || b<=0))	{ Error=MGL_ERR_LOG; return NAN; }
-		return f2[Kod-EQ_LT](a,b);
-	}
-	else if(Kod<EQ_SN)
-	{
-		if((Kod==EQ_ACOS || Kod==EQ_ASIN) && fabs(a)>1)	{ Error=MGL_ERR_ARC; return NAN; }
-		if((Kod==EQ_LG || Kod==EQ_LN) && a<=0)	{ Error=MGL_ERR_LOG; return NAN; }
-		if(Kod==EQ_SQRT && a<0) { Error=MGL_ERR_SQRT; return NAN; }
-		return f1[Kod-EQ_SIN](a);
-	}
-#ifndef NO_GSL
-	else if(Kod<=EQ_DC)
-	{
-		double sn=0,cn=0,dn=0,b = Right->CalcIn(a1);
-		if(isnan(b))	return NAN;
-		gsl_sf_elljac_e(a,b, &sn, &cn, &dn);
-		switch(Kod)
+		if(Kod<EQ_SIN)
 		{
-		case EQ_SN:		return sn;
-		case EQ_SC:		return sn/cn;
-		case EQ_SD:		return sn/dn;
-		case EQ_CN:		return cn;
-		case EQ_CS:		return cn/sn;
-		case EQ_CD:		return cn/dn;
-		case EQ_DN:		return dn;
-		case EQ_DS:		return dn/sn;
-		case EQ_DC:		return dn/cn;
-		case EQ_NS:		return 1./sn;
-		case EQ_NC:		return 1./cn;
-		case EQ_ND:		return 1./dn;
+			double b = Right->CalcIn(a1);
+			return !isnan(b) ? f2[Kod-EQ_LT](a,b) : NAN;
+		}
+		else if(Kod<EQ_SN)	return f1[Kod-EQ_SIN](a);
+#ifndef NO_GSL
+		else if(Kod<=EQ_DC)
+		{
+			double sn=0,cn=0,dn=0,b = Right->CalcIn(a1);
+			if(isnan(b))	return NAN;
+			gsl_sf_elljac_e(a,b, &sn, &cn, &dn);
+			switch(Kod)
+			{
+			case EQ_SN:		return sn;
+			case EQ_SC:		return sn/cn;
+			case EQ_SD:		return sn/dn;
+			case EQ_CN:		return cn;
+			case EQ_CS:		return cn/sn;
+			case EQ_CD:		return cn/dn;
+			case EQ_DN:		return dn;
+			case EQ_DS:		return dn/sn;
+			case EQ_DC:		return dn/cn;
+			case EQ_NS:		return 1./sn;
+			case EQ_NC:		return 1./cn;
+			case EQ_ND:		return 1./dn;
+			}
 		}
-	}
 #endif
-	return Res;
+	}
+	return NAN;
 }
 //-----------------------------------------------------------------------------
 double mgp(double ,double )	{return 1;}
@@ -644,8 +620,8 @@ mreal mglFormula::CalcDIn(int id, const mreal *a1) const
 			,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2
 #endif
 		};
-	func_1 f11[41] = {cos,cos_d,tan_d,asin_d,acos_d,atan_d,cosh,sinh,tanh_d,
-					asinh_d,acosh_d,atanh_d,sqrt_d,exp,log_d,log10_d,mgz1,mgz1,sgn
+	func_1 f11[42] = {cos,cos_d,tan_d,asin_d,acos_d,atan_d,cosh,sinh,tanh_d,
+					asinh_d,acosh_d,atanh_d,sqrt_d,exp,log_d,log10_d,mgz1,mgz1,mgz1,sgn
 #ifndef NO_GSL
 			,dilog_d,gslE_d,gslK_d,gslAi_d,gslBi_d,erf_d,exp3_d,ei_d,e1_d,e2_d,
 			si_d,ci_d,gamma_d,gsl_sf_psi_1,mgz1,mgz1,sinc_d,mgz1,mgz1,mgz1,mgz1,mgz1
@@ -655,51 +631,42 @@ mreal mglFormula::CalcDIn(int id, const mreal *a1) const
 #endif
 		};
 //	if(Error)	return 0;
-	if(Kod==EQ_A)	return id==(int)Res?1:0;
-	if(Kod==EQ_RND || Kod==EQ_NUM) return 0;
+	if(Kod<EQ_LT)	return (Kod==EQ_A && id==(int)Res)?1:0;
 
 	double a = Left->CalcIn(a1), d = Left->CalcDIn(id,a1);
-	if(isnan(a))	return NAN;
-
-	if(Kod<EQ_SIN)
+	if(!isnan(a) && !isnan(d))
 	{
-		double b = Right->CalcIn(a1);
-		if(isnan(b))	return NAN;
-		if(Kod==EQ_POW && a<=0)		{ Error=MGL_ERR_LOG; return NAN; }
-		if(Kod==EQ_LOG && (a<=0 || b<=0))	{ Error=MGL_ERR_LOG; return NAN; }
-		return f21[Kod-EQ_LT](a,b)*d + f22[Kod-EQ_LT](a,b)*Right->CalcDIn(id,a1);
-	}
-	else if(Kod<EQ_SN)
-	{
-		if((Kod==EQ_ACOS || Kod==EQ_ASIN) && fabs(a)>1)	{ Error=MGL_ERR_ARC; return NAN; }
-		if((Kod==EQ_LG || Kod==EQ_LN) && a<=0)	{ Error=MGL_ERR_LOG; return NAN; }
-		if(Kod==EQ_SQRT && a<0) { Error=MGL_ERR_SQRT; return NAN; }
-		return f11[Kod-EQ_SIN](a)*d;
-	}
-#ifndef NO_GSL
-	else if(Kod<=EQ_DC)
-	{
-		double sn=0,cn=0,dn=0,b = Right->CalcIn(a1);
-		if(isnan(b))	return NAN;
-		gsl_sf_elljac_e(a,b, &sn, &cn, &dn);
-		switch(Kod)	// At this moment parse only differentiation or argument NOT mu !!!
+		if(Kod<EQ_SIN)
 		{
-		case EQ_SN:		return cn*dn*d;
-		case EQ_SC:		return dn*d/(cn*cn);
-		case EQ_SD:		return cn*d/(dn*dn);
-		case EQ_CN:		return -dn*sn*d;
-		case EQ_CS:		return dn*d/(sn*sn);
-		case EQ_CD:		return (b-1)*d*sn/(dn*dn);
-		case EQ_DN:		return -b*d*cn*sn;
-		case EQ_DS:		return -cn*d/(sn*sn);
-		case EQ_DC:		return (1-b)*sn*d/(cn*cn);
-		case EQ_NS:		return -cn*dn*d/(sn*sn);
-		case EQ_NC:		return dn*sn*d/(cn*cn);
-		case EQ_ND:		return b*cn*sn*d/(dn*dn);
+			double b = Right->CalcIn(a1);
+			return !isnan(b) ? f21[Kod-EQ_LT](a,b)*d + f22[Kod-EQ_LT](a,b)*Right->CalcDIn(id,a1) : NAN;
+		}
+		else if(Kod<EQ_SN)	return f11[Kod-EQ_SIN](a)*d;
+#ifndef NO_GSL
+		else if(Kod<=EQ_DC)
+		{
+			double sn=0,cn=0,dn=0,b = Right->CalcIn(a1);
+			if(isnan(b))	return NAN;
+			gsl_sf_elljac_e(a,b, &sn, &cn, &dn);
+			switch(Kod)	// At this moment parse only differentiation or argument NOT mu !!!
+			{
+			case EQ_SN:		return cn*dn*d;
+			case EQ_SC:		return dn*d/(cn*cn);
+			case EQ_SD:		return cn*d/(dn*dn);
+			case EQ_CN:		return -dn*sn*d;
+			case EQ_CS:		return dn*d/(sn*sn);
+			case EQ_CD:		return (b-1)*d*sn/(dn*dn);
+			case EQ_DN:		return -b*d*cn*sn;
+			case EQ_DS:		return -cn*d/(sn*sn);
+			case EQ_DC:		return (1-b)*sn*d/(cn*cn);
+			case EQ_NS:		return -cn*dn*d/(sn*sn);
+			case EQ_NC:		return dn*sn*d/(cn*cn);
+			case EQ_ND:		return b*cn*sn*d/(dn*dn);
+			}
 		}
-	}
 #endif
-	return Res;
+	}
+	return NAN;
 }
 //-----------------------------------------------------------------------------
 //   
