@@ -21,6 +21,8 @@
 #include <string.h>
 #include <stdlib.h>
 #include <time.h>
+#include <zlib.h>
+#include <stdarg.h>
 #ifndef NO_PNG
 #include <png.h>
 #endif
@@ -45,6 +47,8 @@ unsigned char **mglGraph::GetRGBLines(long &, long &, unsigned char *&f, bool )
 //-----------------------------------------------------------------------------
 void mglGraph::WriteSVG(const char *,const char *)	{}
 void mglGraph::WriteIDTF(const char *,const char *)	{}
+void mglGraph::WriteU3D(const char *,const char *)	{}
+void mglGraph::WritePDF(const char *,const char *)	{}
 void mglGraph::DoubleSided(bool )	{}
 void mglGraph::VertexColor(bool )	{}
 void mglGraph::TextureColor(bool ){}
@@ -213,9 +217,7 @@ int mgl_jpeg_save(const char *fname, int w, int h, unsigned char **p)
 	cinfo.in_color_space = JCS_RGB;
 	jpeg_set_defaults(&cinfo);
 	jpeg_start_compress(&cinfo, TRUE);
-
-    jpeg_write_scanlines(&cinfo, p, h);
-
+	jpeg_write_scanlines(&cinfo, p, h);
 	jpeg_finish_compress(&cinfo);
 	jpeg_destroy_compress(&cinfo);
 	fclose(outfile);
@@ -223,26 +225,37 @@ int mgl_jpeg_save(const char *fname, int w, int h, unsigned char **p)
 	return 0;
 }
 //-----------------------------------------------------------------------------
+void mgl_printf(void *fp, bool gz, const char *str, ...)
+{
+	char buf[512];
+	va_list lst;
+	va_start(lst,str);
+	vsprintf(buf,str,lst);
+	va_end(lst);
+	if(gz)	gzprintf(fp, "%s", buf);
+	else	fprintf((FILE *)fp, "%s", buf);
+}
+//---------------------------------------------------------------------------
 int mgl_bps_save(const char *fname, int w, int h, unsigned char **p)
 {
 	time_t now;
 	time(&now);
 	register long i,j;
+	bool gz = fname[strlen(fname)-1]=='z';
 
-	FILE *fp = fopen(fname,"wt");
-	fprintf(fp,"%%!PS-Adobe-3.0 EPSF-3.0\n%%%%BoundingBox: 0 0 %d %d\n",w,h);
-	fprintf(fp,"%%%%Creator: MathGL library\n%%%%Title: %s\n", fname);
-	fprintf(fp,"%%%%CreationDate: %s\n",ctime(&now));
-//	fprintf(fp,"%%%%EndComments\n\n");
-	fprintf(fp,"%d %d 8 [1 0 0 1 0 0] {currentfile %d string readhexstring pop} false 3 colorimage\n",
+	void *fp = gz ? gzopen(fname,"wt") : fopen(fname,"wt");
+	mgl_printf(fp, gz, "%%!PS-Adobe-3.0 EPSF-3.0\n%%%%BoundingBox: 0 0 %d %d\n",w,h);
+	mgl_printf(fp, gz, "%%%%Creator: MathGL library\n%%%%Title: %s\n", fname);
+	mgl_printf(fp, gz, "%%%%CreationDate: %s\n",ctime(&now));
+	mgl_printf(fp, gz, "%d %d 8 [1 0 0 1 0 0] {currentfile %d string readhexstring pop} false 3 colorimage\n",
 			w,h,1+w*h/40);
 	for(j=h-1;j>=0;j--)	for(i=0;i<w;i++)
 	{
-		if((i+w*(h-j-1))%40==0 && i+j>0)	fprintf(fp,"\n");
-		fprintf(fp,"%02x%02x%02x",p[j][3*i],p[j][3*i+1],p[j][3*i+2]);
+		if((i+w*(h-j-1))%40==0 && i+j>0)	mgl_printf(fp, gz, "\n");
+		mgl_printf(fp, gz, "%02x%02x%02x",p[j][3*i],p[j][3*i+1],p[j][3*i+2]);
 	}
-	fprintf(fp,"\n\nshowpage\n%%%%EOF\n");
-	fclose(fp);
+	mgl_printf(fp, gz, "\n\nshowpage\n%%%%EOF\n");
+	if(gz)	gzclose(fp);	else	fclose((FILE *)fp);
 	return 0;
 }
 //-----------------------------------------------------------------------------
@@ -378,6 +391,8 @@ void mglGraph::WriteFrame(const char *fname, const char *descr)
 	if(!strcmp(fname+len-4,".jpg"))	WriteJPEG(fname,descr);
 	if(!strcmp(fname+len-5,".jpeg"))WriteJPEG(fname,descr);
 	if(!strcmp(fname+len-5,".idtf"))WriteIDTF(fname,descr);
+	if(!strcmp(fname+len-4,".u3d")) WriteU3D(fname,descr);
+	if(!strcmp(fname+len-4,".pdf")) WritePDF(fname,descr);
 	if(!strcmp(fname+len-4,".png"))	WritePNG(fname,descr,false);
 	if(!strcmp(fname+len-4,".eps"))	WriteEPS(fname,descr);
 	if(!strcmp(fname+len-4,".svg"))	WriteSVG(fname,descr);
@@ -439,6 +454,28 @@ void mgl_write_idtf_(uintptr_t *gr, const char *fname,const char *descr,int l,in
 	_GR_->WriteIDTF(s,f);	delete []s;		delete []f;
 }
 //-----------------------------------------------------------------------------
+/// Write the frame in file using U3D format
+void mgl_write_u3d(HMGL gr, const char *fname,const char *descr)
+{	gr->WriteU3D(fname,descr);	}
+/// Write the frame in file using U3D format
+void mgl_write_u3d_(uintptr_t *gr, const char *fname,const char *descr,int l,int n)
+{
+	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
+	char *f=new char[n+1];	memcpy(f,descr,n);	f[n]=0;
+	_GR_->WriteU3D(s,f);	delete []s;		delete []f;
+}
+//-----------------------------------------------------------------------------
+/// Write the frame in file using PDF format
+void mgl_write_pdf(HMGL gr, const char *fname,const char *descr)
+{	gr->WritePDF(fname,descr);	}
+/// Write the frame in file using PDF format
+void mgl_write_pdf_(uintptr_t *gr, const char *fname,const char *descr,int l,int n)
+{
+	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
+	char *f=new char[n+1];	memcpy(f,descr,n);	f[n]=0;
+	_GR_->WritePDF(s,f);	delete []s;		delete []f;
+}
+//-----------------------------------------------------------------------------
 /// Write the frame in file using PNG format
 void mgl_write_gif(HMGL gr, const char *fname,const char *descr)
 {	gr->WriteGIF(fname,descr);	}
