@@ -183,9 +183,12 @@ void mgl_set_ambbr_(uintptr_t *gr, mreal *i)
 //-----------------------------------------------------------------------------
 //		Scale and rotate
 //-----------------------------------------------------------------------------
+/// Push transformation matrix into stack.
+void mgl_mat_push_(uintptr_t *gr)	{	_GR_->Push();	}
+/// Pop transformation matrix into stack.
+void mgl_mat_pop_(uintptr_t *gr)	{	_GR_->Pop();	}
 /// Clear transformation matrix.
-void mgl_identity_(uintptr_t *gr, int *rel)
-{	_GR_->Identity(*rel);	}
+void mgl_identity_(uintptr_t *gr, int *rel)	{	_GR_->Identity(*rel);	}
 /// Clear up the frame
 void mgl_clf_(uintptr_t *gr)
 {	_GR_->Clf();	}
@@ -195,16 +198,20 @@ void mgl_clf_rgb_(uintptr_t *gr, mreal *r, mreal *g, mreal *b)
 /// Put further plotting in some region of whole frame surface.
 void mgl_subplot_(uintptr_t *gr, int *nx,int *ny,int *m)
 {	_GR_->SubPlot(*nx,*ny,*m);	}
-/// Put further plotting in some region of whole frame surface.
 void mgl_subplot_d_(uintptr_t *gr, int *nx,int *ny,int *m,mreal *dx,mreal *dy)
 {	_GR_->SubPlot(*nx,*ny,*m,*dx,*dy);	}
+void mgl_subplot_s_(uintptr_t *gr, int *nx,int *ny,int *m,const char *st,int l)
+{	char *s=new char[l+1];	memcpy(s,st,l);	s[l]=0;
+	_GR_->SubPlot(*nx,*ny,*m,s);	delete []s;	}
 /// Put further plotting in some region of whole frame surface.
 void mgl_inplot_(uintptr_t *gr, mreal *x1,mreal *x2,mreal *y1,mreal *y2)
-{	_GR_->InPlot(*x1,*x2,*y1,*y2);	}
+{	_GR_->InPlot(*x1,*x2,*y1,*y2,false);	}
 void mgl_relplot_(uintptr_t *gr, mreal *x1,mreal *x2,mreal *y1,mreal *y2)
 {	_GR_->InPlot(*x1,*x2,*y1,*y2,true);	}
 void mgl_columnplot_(uintptr_t *gr, int *num, int *i)
 {	_GR_->ColumnPlot(*num,*i);	}
+void mgl_columnplot_d_(uintptr_t *gr, int *num, int *i, mreal *d)
+{	_GR_->ColumnPlot(*num,*i,*d);	}
 void mgl_stickplot_(uintptr_t *gr, int *num, int *i, mreal *tet, mreal *phi)
 {	_GR_->StickPlot(*num, *i, *tet, *phi);	}
 /// Set aspect ratio for further plotting.
@@ -460,6 +467,8 @@ void mgl_legend_(uintptr_t *gr, int *where, const char *font, mreal *size, mreal
 }
 void mgl_set_legend_box_(uintptr_t *gr, int *enable)
 {	_GR_->LegendBox = *enable;	}
+void mgl_set_legend_marks_(uintptr_t *gr, int *num)
+{	_GR_->SetLegendMarks(*num);	}
 //-----------------------------------------------------------------------------
 const unsigned char *mgl_get_rgb_(uintptr_t *graph)
 {
@@ -507,6 +516,52 @@ void mgl_calc_scr_(uintptr_t *gr, mreal *x, mreal *y, mreal *z, int *xs, int *ys
 	if(g)	g->CalcScr(mglPoint(*x,*y,*z),xs,ys);
 }
 //-----------------------------------------------------------------------------
+void mgl_wnd_set_delay_(uintptr_t *gr, mreal *dt)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->Delay = *dt;	}
+void mgl_wnd_set_auto_clf_(uintptr_t *gr, int *val)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->AutoClf = *val;	}
+void mgl_wnd_set_show_mouse_pos_(uintptr_t *gr, int *val)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->ShowMousePos = *val;	}
+void mgl_wnd_set_clf_update_(uintptr_t *gr, int *val)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->ClfOnUpdate = *val;	}
+void mgl_wnd_toggle_alpha_(uintptr_t *gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->ToggleAlpha();	}
+void mgl_wnd_toggle_light_(uintptr_t *gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->ToggleLight();	}
+void mgl_wnd_toggle_zoom_(uintptr_t *gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->ToggleZoom();	}
+void mgl_wnd_toggle_rotate_(uintptr_t *gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->ToggleRotate();	}
+void mgl_wnd_toggle_no_(uintptr_t *gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->ToggleNo();	}
+void mgl_wnd_update_(uintptr_t *gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->Update();	}
+void mgl_wnd_reload_(uintptr_t *gr, int *o)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->ReLoad(*o);	}
+void mgl_wnd_adjust_(uintptr_t *gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->Adjust();	}
+void mgl_wnd_next_frame_(uintptr_t *gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->NextFrame();	}
+void mgl_wnd_prev_frame_(uintptr_t *gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->PrevFrame();	}
+void mgl_wnd_animation_(uintptr_t *gr)
+{	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->Animation();	}
+//-----------------------------------------------------------------------------
 mreal mgl_data_get_value_(uintptr_t *d, int *i, int *j, int *k)
 {	return	_DT_->a[*i+_DT_->nx*(*j+*k*_DT_->ny)];	}
 void mgl_data_set_value_(uintptr_t *d, mreal *v, int *i, int *j, int *k)
@@ -581,4 +636,5 @@ void mgl_set_tick_stl_(uintptr_t *gr, const char *stl, const char *sub, int l, i
 	_GR_->SetTickStl(t,s);
 	delete []s;	delete []t;
 }
+int mgl_get_warn_(uintptr_t *gr)	{	return _GR_->WarnCode;	}
 //-----------------------------------------------------------------------------
