@@ -35,6 +35,41 @@ wchar_t *wcstokw32(wchar_t *wcs, const wchar_t *delim)	{	return wcstok(wcs,delim
 
 wchar_t *mgl_wcsdup(const wchar_t *s);
 //-----------------------------------------------------------------------------
+mglFunc::mglFunc(long p, const wchar_t *f, mglFunc *prev)
+{
+	pos = p;	next = prev;
+	register long i;
+	for(i=0;(isalnum(f[i]) || f[i]=='_') && i<31;i++)	func[i]=f[i];
+	func[i]=0;
+	narg = wcstol(f+i+1,0,0);
+	if(narg<0 || narg>9)	narg=0;
+}
+//-----------------------------------------------------------------------------
+long mglParse::IsFunc(const wchar_t *name, int *na)
+{
+	mglFunc *f=func;
+	while(f)
+	{
+		if(!wcscmp(name,f->func))
+		{	if(na)	*na=f->narg;	return f->pos;	}
+		f = f->next;
+	}
+	return 0;
+}
+//-----------------------------------------------------------------------------
+void mglParse::ScanFunc(const wchar_t *line)
+{
+	static long num=0;
+	if(!line)
+	{	if(func)	delete func;
+		num=0;	func=0;	return;	}
+	num++;
+	if(wcsncmp(line,L"func",4) || line[4]>' ')	return;
+	register long i;
+	for(i=4;line[i]<=' ' || line[i]=='\'';i++);
+	func = new mglFunc(num-1, line+i, func);
+}
+//-----------------------------------------------------------------------------
 void mgl_wcstrim(wchar_t *str)
 {
 	wchar_t *c = mgl_wcsdup(str);
@@ -160,7 +195,7 @@ mglParse::mglParse(bool setsize)
 	AllowSetSize=setsize;
 	Once = true;	parlen=320;
 	op1 = new wchar_t[4096];	op2 = new wchar_t[4096];
-	fval = new mglData[10];
+	fval = new mglData[40];
 }
 //-----------------------------------------------------------------------------
 mglParse::~mglParse()
@@ -170,9 +205,10 @@ mglParse::~mglParse()
 		while(DataList->next)	delete DataList->next;
 		delete DataList;
 	}
-	for(long i=0;i<10;i++)	if(par[i])	delete [] par[i];
+	for(long i=0;i<40;i++)	if(par[i])	delete []par[i];
 	delete []op1;	delete []op2;	delete []fval;
 	if(Cmd!=mgls_base_cmd)	delete []Cmd;
+	if(fn_stack)	free(fn_stack);
 }
 //-----------------------------------------------------------------------------
 bool mglParse::AddParam(int n, const char *str, bool isstr)
@@ -235,7 +271,7 @@ mglNum *mglParse::FindNum(const char *str)
 //-----------------------------------------------------------------------------
 bool mglParse::AddParam(int n, const wchar_t *str, bool isstr)
 {
-	if(n<0 || n>9 || wcschr(str,'$'))	return false;
+	if(n<0 || n>39 || wcschr(str,'$'))	return false;
 	if(!isstr)	parlen += wcslen(str);
 	if(par[n])	delete []par[n];
 	par[n] = new wchar_t[wcslen(str)+1];
@@ -474,11 +510,32 @@ int mglParse::PreExec(mglGraph *, long k, wchar_t **arg, mglArg *a)
 	return n;
 }
 //-----------------------------------------------------------------------------
+void mglParse::PutArg(const wchar_t *string, wchar_t *str, bool def)
+{
+	if(parlen>0)
+	{
+		wchar_t *sb = new wchar_t[wcslen(string)+1], *t;
+		if(def)	str = str+10;
+		register long n;
+		while((t=wcschr(str,'$'))!=0)
+		{
+			wcscpy(sb,t+2);
+			t[0]=0;
+			n = t[1]-'0';	if(n>=0 && n<=9 && par[n])	wcscat(str,par[n]);
+			n = t[1]-'a';	if(n>=0 && n<='z'-'a' && par[n+10])	wcscat(str,par[n+10]);
+			if(t[1]=='$')	wcscat(str,L"\xffff");
+			wcscat(str,sb);
+		}
+		delete []sb;
+		while((t=wcschr(str,L'\xffff'))!=0)	*t='$';
+	}
+}
+//-----------------------------------------------------------------------------
 // return values: 0 - OK, 1 - wrong arguments, 2 - wrong command, 3 - string too long, 4 -- unclosed string
 int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 {
 	if(!gr || Stop)	return 0;
-	wchar_t *str, *s = new wchar_t[wcslen(string)+1+parlen],*arg[1024],*t;
+	wchar_t *str, *s = new wchar_t[wcslen(string)+1+40*parlen],*arg[1024],*t;
 	str = s;
 	wcscpy(str,string);
 	wcstrim_mgl(str);
@@ -499,13 +556,13 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 		}
 	}
 	// check if string is closed
-	for(n=1,k=0;n<long(wcslen(str));n++)
-		if(str[n]=='\'' && str[n-1]!='\\')	k++;
+	for(n=1,k=0;n<long(wcslen(str));n++)	if(str[n]=='\'' && str[n-1]!='\\')	k++;
 	if(k%2)	return 4;	// strings is not closed
 	// define parameters or start cycle
 	if(!skip() && !wcsncmp(str,L"def",3) && (str[6]==' ' || str[6]=='\t'))
 	{
-		if(!skip() && !wcsncmp(str+3,L"ine",3) && (str[6]==' ' || str[6]=='\t'))
+		PutArg(string,str,true);
+		if(!wcsncmp(str+3,L"ine",3))
 		{
 			wchar_t *ss=str+7;	wcstrim_mgl(ss);//	int res = 1;
 			if(*ss=='$' && ss[1]>='0' && ss[1]<='9')
@@ -515,8 +572,15 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 				AddParam(n, ss);
 				delete []s;	return 0;
 			}
+			if(*ss=='$' && ss[1]>='a' && ss[1]<='z')
+			{
+				int n=ss[1]-'a';//	res = 0;
+				ss +=2;	mgl_wcstrim(ss);
+				AddParam(n+10, ss);
+				delete []s;	return 0;
+			}
 		}
-		if(!skip() && !wcsncmp(str+3,L"num",3) && (str[6]==' ' || str[6]=='\t'))
+		if(!wcsncmp(str+3,L"num",3))
 		{
 			str += 7;	wcstrim_mgl(str);	int res = 1;
 			if(*str=='$' && str[1]>='0' && str[1]<='9')
@@ -529,9 +593,19 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 				AddParam(n, buf);
 				delete []buf;
 			}
+			if(*str=='$' && str[1]>='a' && str[1]<='z')
+			{
+				int n=str[1]-'a';	res = 0;
+				str +=2;	mgl_wcstrim(str);
+				const mglData &d=mglFormulaCalc(str, this);
+				char *buf=new char[128];
+				sprintf(buf,"%g",d.a[0]);
+				AddParam(n+10, buf);
+				delete []buf;
+			}
 			delete []s;		return res;
 		}
-		if(!skip() && !wcsncmp(str+3,L"chr",3) && (str[6]==' ' || str[6]=='\t'))
+		if(!wcsncmp(str+3,L"chr",3))
 		{
 			str += 7;	wcstrim_mgl(str);	int res = 1;
 			if(*str=='$' && str[1]>='0' && str[1]<='9')
@@ -542,9 +616,17 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 				wchar_t buf[2]={0,0};	buf[0] = wchar_t(d.a[0]);
 				AddParam(n, buf);
 			}
+			if(*str=='$' && str[1]>='a' && str[1]<='z')
+			{
+				int n=str[1]-'a';	res = 0;
+				str +=2;	mgl_wcstrim(str);
+				const mglData &d=mglFormulaCalc(str, this);
+				wchar_t buf[2]={0,0};	buf[0] = wchar_t(d.a[0]);
+				AddParam(n+10, buf);
+			}
 			delete []s;		return res;
 		}
-		if(!skip() && !wcsncmp(str+3,L"pal",3) && (str[6]==' ' || str[6]=='\t'))
+		if(!wcsncmp(str+3,L"pal",3))
 		{
 			str += 7;	wcstrim_mgl(str);	int res = 1;
 			if(*str=='$' && str[1]>='0' && str[1]<='9')
@@ -556,6 +638,15 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 				buf[0] = gr->DefPal[int(d.a[0])%gr->NumPal];
 				AddParam(n, buf);
 			}
+			if(*str=='$' && str[1]>='a' && str[1]<='z')
+			{
+				int n=str[1]-'a';	res = 0;
+				str +=2;	mgl_wcstrim(str);
+				const mglData &d=mglFormulaCalc(str, this);
+				wchar_t buf[2]={0,0};
+				buf[0] = gr->DefPal[int(d.a[0])%gr->NumPal];
+				AddParam(n+10, buf);
+			}
 			delete []s;		return res;
 		}
 	}
@@ -565,24 +656,10 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 		while(*t && isspace(*t))	t++;
 		// if command have format 'for $N ...' then change it to 'for N ...'
 		if(*t=='$' && t[1]>='0' && t[1]<='9')	*t = ' ';
+		if(*t=='$' && t[1]>='a' && t[1]<='z')	*t = ' ';
 	}
 	// parse arguments (parameters $1, ..., $9)
-	if(parlen>0)
-	{
-		wchar_t *sb = new wchar_t[wcslen(string)+1];
-		while((t=wcschr(str,'$'))!=0)
-		{
-			wcscpy(sb,t+2);
-			t[0]=0;
-			long n = t[1]-'0';
-			if(n>=0 && n<=9 && par[n])	wcscat(str,par[n]);
-			wcscat(str,sb);
-		}
-		delete []sb;
-	}
-
-	wcstrim_mgl(str);
-	if(!skip() && !wcscmp(str,L"stop"))	{	Stop = true;	delete []s;	return 0;	}
+	PutArg (string,str,false);	wcstrim_mgl(str);
 
 	for(k=0;k<1024;k++)	// parse string to substrings (by spaces)
 	{
@@ -605,6 +682,9 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 		mglArg *a = new mglArg[k];
 		FillArg(gr, k, arg, a);
 		// execute first special (program-flow-control) commands
+		if(!skip() && !wcscmp(arg[0],L"stop"))
+		{	Stop = true;	delete []s;	delete []a;	return 0;	}
+		if(!wcscmp(arg[0],L"func"))	{	delete []s;	delete []a;	return 0;	}
 		n = FlowExec(gr, arg[0],k-1,a);
 		if(n)		{	delete []s;	delete []a;	return n-1;	}
 		if(skip())	{	delete []s;	delete []a;	return 0;	}
@@ -623,16 +703,45 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 			n = 1;
 			if(a[0].type==1)
 			{
-				n=0;	mgl_wcstombs(a[0].s, a[0].w, 1024);
-				FILE *fp = fopen(a[0].s,"rt");	Execute(gr,fp);	fclose(fp);
+				int na=0;
+				mgl_wcstombs(a[0].s, a[0].w, 1024);		n=-IsFunc(a[0].w,&na);
+				if(n && k!=na+2)	
+				{
+					if(gr->Message)
+						sprintf(gr->Message,"Bad arguments for %ls: %ld instead of %d\n",
+								a[0].w,k-2,na);
+					n = 1;
+				}
+				else if(n)
+				{
+					if(!fn_stack)
+					{	fn_num = 100;
+						fn_stack = (mglFnStack*)malloc(fn_num*sizeof(mglFnStack));	}
+					if(fn_pos >= fn_num)
+					{	fn_num+= 100;
+						fn_stack = (mglFnStack*)realloc(fn_stack,fn_num*sizeof(mglFnStack));	}
+					memcpy(fn_stack[fn_pos].par,par+1,9*sizeof(wchar_t*));
+					memset(par+1,0,9*sizeof(wchar_t*));
+					for(int i=1;i<k-1;i++)	AddParam(i,arg[i+1]);
+					fn_stack[fn_pos].pos = pos;	fn_pos++;	n--;
+				}
+				else
+				{
+					FILE *fp = fopen(a[0].s,"rt");
+					if(fp)	{	Execute(gr,fp);	fclose(fp);	}
+					else	n=1;
+				}
 			}
 			delete []s;	delete []a;	return n;
 		}
 		if(!wcscmp(arg[0],L"for"))
 		{
 			n = 1;
-			int r = int(a[0].v);
-			if(a[0].type==2 && (r>=0 || r<=9))
+			char ch = arg[1][0];
+			int r = ch-'0';
+			if(ch>='a' && ch<='z')	r = 10+ch-'a';
+//			int r = int(a[0].v);
+			if(arg[1][1]==0 && (r>=0 || r<=39))
 			{
 				if(a[1].type==0)
 				{
@@ -652,7 +761,7 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 				}
 				if(n==0)
 				{
-					for(int i=9;i>0;i--)
+					for(int i=39;i>0;i--)
 					{	for_stack[i]=for_stack[i-1];	if_for[i]=if_for[i-1];	}
 					for_stack[0] = r+1;		fval[r].nz = pos;	if_for[0]=if_pos;
 					wchar_t buf[32];		mglprintf(buf,32,L"%g",fval[r].a[0]);
@@ -755,7 +864,7 @@ int mglParse::FlowExec(mglGraph *, const wchar_t *com, long m, mglArg *a)
 		}
 		else n = 1;
 		if(n==0)
-		{	if_stack[if_pos] = cond;	if_pos = if_pos<19 ? if_pos+1 : 19;	}
+		{	if_stack[if_pos] = cond;	if_pos = if_pos<39 ? if_pos+1 : 39;	}
 	}
 	else if(!Skip && !wcscmp(com,L"endif"))
 	{
@@ -796,6 +905,13 @@ int mglParse::FlowExec(mglGraph *, const wchar_t *com, long m, mglArg *a)
 		if(if_pos==if_for[0])	if_pos = if_pos>0 ? if_pos-1 : 0;
 		if(out)	mglprintf(out,1024,L"break;");	for_br = true;
 	}
+	else if(!skip() && !wcscmp(com, L"return"))	// parse command "delete"
+	{
+		if(!fn_pos)	return 2;
+		fn_pos--;	n = -fn_stack[fn_pos].pos-1;
+		for(int i=1;i<10;i++)	if(par[i])	delete []par[i];
+		memcpy(par+1,fn_stack[fn_pos].par,9*sizeof(wchar_t*));
+	}
 	else if(!ifskip() && !Skip && !wcscmp(com,L"next"))
 	{
 		if(if_pos==if_for[0])	if_pos = if_pos>0 ? if_pos-1 : 0;
@@ -811,9 +927,9 @@ int mglParse::FlowExec(mglGraph *, const wchar_t *com, long m, mglArg *a)
 			}
 			else
 			{
-				for(int i=0;i<9;i++)
+				for(int i=0;i<39;i++)
 				{	for_stack[i]=for_stack[i+1];	if_for[i]=if_for[i+1];	}
-				for_stack[9] = 0;	for_br=false;
+				for_stack[39] = 0;	for_br=false;
 			}
 		}
 		if(out)	mglprintf(out,1024,L"}");
@@ -845,7 +961,7 @@ void mgl_error_print(int line, int r, mglGraph *gr)
 	if(r==2)	printf("Wrong command in line %d\n", line);
 	if(r==3)	printf("String too long in line %d\n", line);
 	if(r==4)	printf("Unbalanced ' in line %d\n", line);
-	gr->Message[0]=0;
+	if(gr->Message)	gr->Message[0]=0;
 }
 void mglParse::Execute(mglGraph *gr, FILE *fp, bool print)
 {
@@ -868,14 +984,16 @@ void mglParse::Execute(mglGraph *gr, int n, const wchar_t **text, void (*error)(
 {
 	if(gr==0 || n<1 || text==0)	return;
 	long i, r;
-	for_br=Skip=false;	if_pos=0;
+	for_br=Skip=false;	if_pos=fn_pos=0;	ScanFunc(0);
+	for(i=0;i<n;i++)	ScanFunc(text[i]);
 	for(i=0;i<n;i++)
 	{
+//printf("line %ld : %ls\n",i,text[i]);
 		r = Parse(gr,text[i],i+1);
 		if(r<0)	{	i = -r-2;	continue;	}
 		if(error)
 		{
-			if(r>0)	error(i, r, gr);
+			if(r>0)	error(i+1, r, gr);
 			if(gr->Message && gr->Message[0])	error(i,0,gr);
 		}
 	}
@@ -1127,7 +1245,7 @@ void mglParse::AddCommand(mglCommand *cmd, int mc)
 //-----------------------------------------------------------------------------
 mglCommand mglParse::Prg[]={
 	{L"break",L"Break for-cycle",L"break", 0, 0, 0, 5},
-	{L"call",L"Execute script in external file",L"call 'filename'", 0, 0, 0, 5},
+	{L"call",L"Execute script in external file",L"call 'name' [args]", 0, 0, 0, 5},
 	{L"continue",L"Skip commands and iterate for-cycle again",L"continue", 0, 0, 0, 5},
 	{L"defchr",L"Define parameter as character",L"defchr $N val", 0, 0, 0, 5},
 	{L"define",L"Define constant or parameter",L"define $N sth | Var val", 0, 0, 0, 5},
@@ -1138,9 +1256,11 @@ mglCommand mglParse::Prg[]={
 	{L"elseif",L"Conditional operator",L"elseif val|Dat ['cond']", 0, 0, 0, 5},
 	{L"endif",L"Finish if/else block",L"endif", 0, 0, 0, 5},
 	{L"for",L"For cycle",L"for $N v1 v2 [dv] | $N Dat", 0, 0, 0, 5},
+	{L"func",L"Start function definition and stop execution of main script",L"func 'name' [narg]", 0, 0, 0, 5},
 	{L"if",L"Conditional operator",L"if val|Dat ['cond']", 0, 0, 0, 5},
 	{L"next",L"Start next for-cycle iteration",L"next", 0, 0, 0, 5},
 	{L"once",L"Start/close commands which should executed only once",L"once val", 0, 0, 0, 5},
+	{L"return",L"Return from function",L"return", 0, 0, 0, 5},
 	{L"stop",L"Stop execution",L"stop", 0, 0, 0, 5},
 {L"",0,0, 0, 0, 0, 0}};
 //-----------------------------------------------------------------------------
@@ -1156,6 +1276,7 @@ void mgl_parse_text(HMGL gr, HMPR p, const char *str)	{	p->Execute(gr, str);	}
 void mgl_parsew_text(HMGL gr, HMPR p, const wchar_t *str){	p->Parse(gr, str);	}
 void mgl_restore_once(HMPR p)	{	p->RestoreOnce();	}
 void mgl_parser_allow_setsize(HMPR p, int a)	{	p->AllowSetSize = a;	}
+void mgl_scan_func(HMPR p, const wchar_t *line)	{	p->ScanFunc(line);	}
 //-----------------------------------------------------------------------------
 uintptr_t mgl_create_parser_()	{	return uintptr_t(new mglParse);	}
 void mgl_delete_parser_(uintptr_t* p)	{	delete _PR_;	}
