@@ -29,7 +29,7 @@
 #include <gsl/gsl_fft_complex.h>
 #endif
 //-----------------------------------------------------------------------------
-// Solving equation du/dz = ham(p,q,x,y,z,|u|)[u] where p=d/dx, q=d/dy. At this moment simplified form of ham is supported: ham = f(p,q,z) + g(x,y,z,'u'), where variable 'u'=|u| (for allowing solve nonlinear problems). You may specify imaginary part like ham = p^2 + i*x*(x>0) but only if dependence on variable 'i' is linear (i.e. ham = hre+i*him).
+// Solve equation du/dz = ham(p,q,x,y,z,|u|)[u] where p=d/dx, q=d/dy. At this moment simplified form of ham is supported: ham = f(p,q,z) + g(x,y,z,'u'), where variable 'u'=|u| (for allowing solve nonlinear problems). You may specify imaginary part like ham = p^2 + i*x*(x>0) but only if dependence on variable 'i' is linear (i.e. ham = hre+i*him).
 mglData mglPDE(const char *ham, const mglData &ini_re, const mglData &ini_im, mglPoint Min, mglPoint Max, mreal dz, mreal k0)
 {
 	mglData res;
@@ -41,7 +41,9 @@ mglData mglPDE(const char *ham, const mglData &ini_re, const mglData &ini_im, mg
 #ifndef NO_GSL
 	mglFormula eqs(ham);
 	dual *a = new dual[4*nx*ny], h, h0, h1, h2;	// Add "damping" area
+	mreal *dmp = new mreal[4*nx*ny];
 	memset(a,0,4*nx*ny*sizeof(dual));
+	memset(dmp,0,4*nx*ny*sizeof(mreal));
 	register int i,j,k,i0;
 	for(j=0;j<ny;j++)	for(i=0;i<nx;i++)	// Initial conditions
 	{
@@ -49,9 +51,17 @@ mglData mglPDE(const char *ham, const mglData &ini_re, const mglData &ini_im, mg
 		a[i0] = dual(ini_re.a[i+nx*j], ini_im.a[i+nx*j]);
 		res.a[nz*(i+nx*j)] = abs(a[i0]);
 	}
+	for(j=0;j<2*ny;j++)	for(i=0;i<2*nx;i++)	// mask
+	{
+		i0 = i+2*nx*j;
+		if(i<nx/2)		dmp[i0] = GAMMA*mgl_ipow((nx/2-i)/(nx/2.),2);
+		if(i>3*nx/2)	dmp[i0] = GAMMA*mgl_ipow((i-3*nx/2-1)/(nx/2.),2);
+		if(j<ny/2)		dmp[i0]+= GAMMA*mgl_ipow((ny/2-j)/(ny/2.),2);
+		if(j>3*ny/2)	dmp[i0]+= GAMMA*mgl_ipow((j-3*ny/2-1)/(ny/2.),2);
+	}
 	mreal dx = (Max.x-Min.x)/(nx-1), dy = ny>1?(Max.y-Min.y)/(ny-1):0;
 	mreal dp = M_PI/(Max.x-Min.x)/k0, dq = M_PI/(Max.y-Min.y)/k0;
-	mreal var[MGL_VS], xs=(Min.x+Max.x)/2, ys=(Min.y+Max.y)/2, tmp;
+	mreal var[MGL_VS], xs=(Min.x+Max.x)/2, ys=(Min.y+Max.y)/2;
 	double xx = Min.x - dx*nx/2, yy = Min.x - dy*ny/2;
 	double ff = ny>1?4*nx*ny:2*nx, dd = k0*dz;
 	memset(var,0,MGL_VS*sizeof(mreal));
@@ -70,19 +80,14 @@ mglData mglPDE(const char *ham, const mglData &ini_re, const mglData &ini_im, mg
 			var['y'-'a'] = yy+dy*j;		var['q'-'a'] = 0;
 			var['u'-'a'] = abs(a[i0]);
 			h = dual(-eqs.CalcD(var,'i'), eqs.Calc(var))*dd;
-			tmp = 0;
-			if(i<nx/2)		tmp += GAMMA*mgl_ipow((nx/2-i)/(nx/2.),2);
-			if(i>3*nx/2)	tmp += GAMMA*mgl_ipow((i-3*nx/2-1)/(nx/2.),2);
-			if(j<ny/2)		tmp += GAMMA*mgl_ipow((ny/2-j)/(ny/2.),2);
-			if(j>3*ny/2)	tmp += GAMMA*mgl_ipow((j-3*ny/2-1)/(ny/2.),2);
-			a[i0] *= exp(h)*exp(-double(tmp*dz));
+			a[i0] *= exp(h)*exp(-double(dmp[i0]*dz));
 		}
 		// "central" point
 		var['x'-'a'] = xs;	var['y'-'a'] = ys;
 		var['u'-'a'] = var['p'-'a'] = var['q'-'a'] = 0;
 		h0 = dual(-eqs.CalcD(var,'i'), eqs.Calc(var))*dd;
 		// do fft
-		for(i=0;i<ny;i++)
+		for(i=0;i<2*ny;i++)
 			gsl_fft_complex_transform((double *)(a+i*2*nx), 1, 2*nx, wtx, wsx, forward);
 		if(ny>1) for(j=0;j<2*ny;j++)	for(i=0;i<2*nx;i++)	// step 3/2
 		{
@@ -98,7 +103,7 @@ mglData mglPDE(const char *ham, const mglData &ini_re, const mglData &ini_im, mg
 			h = dual(-eqs.CalcD(var,'i'), eqs.Calc(var))*dd;
 			a[i0] *= exp(h-h1-h2+h0);
 		}
-		if(ny>1) for(i=0;i<nx;i++)
+		if(ny>1) for(i=0;i<2*nx;i++)
 			gsl_fft_complex_transform((double *)(a+i), 2*nx, 2*ny, wty, wsy, forward);
 		for(j=0;j<2*ny;j++)	for(i=0;i<2*nx;i++)	// step 2
 		{
@@ -109,7 +114,7 @@ mglData mglPDE(const char *ham, const mglData &ini_re, const mglData &ini_im, mg
 			a[i0] *= exp(h-h0)/ff;
 		}
 		// do ifft
-		if(ny>1) for(i=0;i<nx;i++)
+		if(ny>1) for(i=0;i<2*nx;i++)
 			gsl_fft_complex_transform((double *)(a+i), 2*nx, 2*ny, wty, wsy, backward);
 		if(ny>1) for(j=0;j<2*ny;j++)	for(i=0;i<2*nx;i++)	// step 3/2
 		{
@@ -125,7 +130,7 @@ mglData mglPDE(const char *ham, const mglData &ini_re, const mglData &ini_im, mg
 			h = dual(-eqs.CalcD(var,'i'), eqs.Calc(var))*dd;
 			a[i0] *= exp(h-h1-h2+h0);
 		}
-		for(i=0;i<ny;i++)
+		for(i=0;i<2*ny;i++)
 			gsl_fft_complex_transform((double *)(a+2*i*nx), 1, 2*nx, wtx, wsx, backward);
 		// save result
 		for(i=0;i<nx;i++)	for(j=0;j<ny;j++)
@@ -135,7 +140,7 @@ mglData mglPDE(const char *ham, const mglData &ini_re, const mglData &ini_im, mg
 	gsl_fft_complex_wavetable_free(wtx);
 	gsl_fft_complex_workspace_free(wsy);
 	gsl_fft_complex_wavetable_free(wty);
-	delete []a;
+	delete []a;	delete []dmp;
 #endif
 	return res;
 }
