@@ -159,7 +159,7 @@ mglTeXsymb mgl_tex_symb[] = {
 	{0x2144, L"Yup"},
 	{0x1b5, L"Zbar"},
 	{0x396, L"Zeta"},
-	{0x2c6, L"^"},
+	{0x5e, L"^"},
 	{0xe5, L"aa"},
 	{0x223e, L"ac"},
 	{0x23e6, L"accurrent"},
@@ -174,7 +174,7 @@ mglTeXsymb mgl_tex_symb[] = {
 	{0xe6, L"ae"},
 	{0x2135, L"aleph"},
 	{0x3b1, L"alpha"},
-	{0x2210, L"amalg"},
+//	{0x2210, L"amalg"},
 	{0x2a3f, L"amalg"},
 	{0x299f, L"angdnr"},
 	{0x2220, L"angle"},
